/*
 * Tencent is pleased to support the open source community by making  XiaoweiSDK Demo Codes available.
 *
 * Copyright (C) 2017 THL A29 Limited, a Tencent company. All rights reserved.
 *
 * Licensed under the MIT License (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://opensource.org/licenses/MIT
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 *
 */
#include "CommonDef.h"

#ifdef __cplusplus
extern "C" {
#endif

extern jobject tx_service;
extern jclass s_clsBinderInfo;

jobjectArray ConverBinderList2JobjectArray(JNIEnv *env, const DEVICE_BINDER_INFO *pBinderList, int nCount) {
    if (!env) {
        return NULL;
    }

    if (NULL == s_clsBinderInfo) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER,
                            "ConverBinderList2JobjectArray NULL == s_clsBinderInfo");
        return NULL;
    }

    jclass clsBinderInfo = s_clsBinderInfo;
    jobject obj = NULL;
    jfieldID userName = env->GetFieldID(clsBinderInfo, "userName", "Ljava/lang/String;");
    jfieldID nickName = env->GetFieldID(clsBinderInfo, "nickName", "[B");
    jfieldID remark = env->GetFieldID(clsBinderInfo, "remark", "[B");
    jfieldID headImgUrl = env->GetFieldID(clsBinderInfo, "headImgUrl", "Ljava/lang/String;");
    jfieldID exportId = env->GetFieldID(clsBinderInfo, "exportId", "Ljava/lang/String;");
    jfieldID sex = env->GetFieldID(clsBinderInfo, "sex", "I");
    jfieldID type = env->GetFieldID(clsBinderInfo, "type", "I");
    jmethodID init = env->GetMethodID(clsBinderInfo, "<init>", "()V");

    jobjectArray arrayBinder = env->NewObjectArray(nCount, clsBinderInfo, NULL);
    if (pBinderList && nCount > 0) {
        for (int i = 0; i < nCount; ++i) {
            obj = env->NewObject(clsBinderInfo, init);
            env->SetIntField(obj, type, (jint) pBinderList[i].type);
            env->SetIntField(obj, sex, (jint) pBinderList[i].sex);
            jstring j_user_name;
            ConvChar2JString(env, pBinderList[i].username, j_user_name);
            env->SetObjectField(obj, userName, j_user_name);
            jbyteArray bytesArr = env->NewByteArray(strlen(pBinderList[i].nickname));
            char* nickNameChar = new char[256];
            strcpy(nickNameChar, pBinderList[i].nickname);
            env->SetByteArrayRegion(bytesArr, 0, strlen(pBinderList[i].nickname), reinterpret_cast<jbyte*>(nickNameChar));
            env->SetObjectField(obj, nickName, bytesArr);
            jbyteArray bytesArr2 = env->NewByteArray(strlen(pBinderList[i].remark));
            char* remarkChar = new char[256];
            strcpy(remarkChar, pBinderList[i].remark);
            env->SetByteArrayRegion(bytesArr2, 0, strlen(pBinderList[i].remark), reinterpret_cast<jbyte*>(remarkChar));
            env->SetObjectField(obj, remark, bytesArr2);
            jstring j_head_url;
            ConvChar2JString(env, pBinderList[i].head_img, j_head_url);
            env->SetObjectField(obj, headImgUrl, j_head_url);
            jstring j_export_id;
            ConvChar2JString(env, pBinderList[i].export_id, j_export_id);
            env->SetObjectField(obj, exportId, j_export_id);
            env->SetObjectArrayElement(arrayBinder, i, obj);
            env->DeleteLocalRef(obj);
            env->DeleteLocalRef(j_user_name);
            env->DeleteLocalRef(bytesArr);
            env->DeleteLocalRef(bytesArr2);
            env->DeleteLocalRef(j_head_url);
            env->DeleteLocalRef(j_export_id);
            delete[] nickNameChar;
            delete[] remarkChar;
        }
    }

    return arrayBinder;
}

/**
 * 绑定者列表变化了
 * @param error 非0表示查询列表失败，主要的失败在于内存空间不足，即pBinderList太小了
 * @param pBinderList
 * @param nCount
 */
void device_on_binder_list_change(int error, const DEVICE_BINDER_INFO *pBinderList, int nCount) {
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "device_on_binder_list_change: nCount[%d]", nCount);
    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER,
                            "device_on_binder_list_change NULL == tx_service");
        return;
    }
    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env) return;

    jobjectArray arrayBinder = NULL;
    if (error == error_null) {
        arrayBinder = ConverBinderList2JobjectArray(env, pBinderList, nCount);
    }                         
    jclass cls = env->GetObjectClass(tx_service);
    jmethodID methodID = env->GetMethodID(cls, "onBinderListChange",
                                          "(I[Lcom/tencent/xiaowei/info/DeviceBinderInfo;)V");
    if (methodID) {
        env->CallVoidMethod(tx_service, methodID, error, arrayBinder);
    }

    env->DeleteLocalRef(cls);
    if (arrayBinder) env->DeleteLocalRef(arrayBinder);

    if (needRelease) Util_ReleaseEnv();
}

void device_on_binder_verify(DEVICE_BINDER_INFO info) {
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "device_on_binder_verify");
    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER,
                            "device_on_binder_verify NULL == tx_service");
        return;
    }

    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env) return;

    jclass clsBinderInfo = s_clsBinderInfo;
    jfieldID userName = env->GetFieldID(clsBinderInfo, "userName", "Ljava/lang/String;");
    jfieldID nickName = env->GetFieldID(clsBinderInfo, "nickName", "[B");
    jfieldID remark = env->GetFieldID(clsBinderInfo, "remark", "[B");
    jfieldID headImgUrl = env->GetFieldID(clsBinderInfo, "headImgUrl", "Ljava/lang/String;");
    jfieldID exportId = env->GetFieldID(clsBinderInfo, "exportId", "Ljava/lang/String;");
    jfieldID sex = env->GetFieldID(clsBinderInfo, "sex", "I");
    jmethodID init = env->GetMethodID(clsBinderInfo, "<init>", "()V");

    jobject obj = env->NewObject(clsBinderInfo, init);
    env->SetIntField(obj, sex, (jint) info.sex);
    jstring j_user_name;
    ConvChar2JString(env, info.username, j_user_name);
    env->SetObjectField(obj, userName, j_user_name);
    jbyteArray bytesArr = env->NewByteArray(strlen(info.nickname));
    env->SetByteArrayRegion(bytesArr, 0, strlen(info.nickname), reinterpret_cast<jbyte*>(info.nickname));
    env->SetObjectField(obj, nickName, bytesArr);
    jbyteArray bytesArr2 = env->NewByteArray(strlen(info.remark));
    env->SetByteArrayRegion(bytesArr2, 0, strlen(info.remark), reinterpret_cast<jbyte*>(info.remark));
    env->SetObjectField(obj, remark, bytesArr2);
    jstring j_head_url;
    ConvChar2JString(env, info.head_img, j_head_url);
    env->SetObjectField(obj, headImgUrl, j_head_url);
    jstring j_export_id;
    ConvChar2JString(env, info.export_id, j_export_id);

    jclass cls = env->GetObjectClass(tx_service);
    jmethodID methodID = env->GetMethodID(cls, "onBinderVerify",
                                          "(Lcom/tencent/xiaowei/info/DeviceBinderInfo;)V");
    if (methodID) {
        env->CallVoidMethod(tx_service, methodID, obj);
    }
       
    env->DeleteLocalRef(j_user_name);
    env->DeleteLocalRef(bytesArr);
    env->DeleteLocalRef(bytesArr2);
    env->DeleteLocalRef(j_head_url);
    env->DeleteLocalRef(j_export_id);
    env->DeleteLocalRef(cls);
    env->DeleteLocalRef(obj);
    if (needRelease) Util_ReleaseEnv();
}

void device_on_qr_code_refresh(int errCode, const char *qr_code_path, unsigned int expireTime, unsigned int expireIn)
{
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "device_on_qr_code_refresh");
    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER,
                            "device_on_qr_code_refresh NULL == tx_service");
        return;
    }

    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env) return;

    jclass cls = env->GetObjectClass(tx_service);
    jmethodID methodID = env->GetMethodID(cls, "onQrCodeRefresh",
                                          "(Ljava/lang/String;II)V");
    if (methodID) {
        jstring path;
        ConvChar2JString(env, qr_code_path, path);
        env->CallVoidMethod(tx_service, methodID, path, expireTime, expireIn);
        env->DeleteLocalRef(path);
    }
    env->DeleteLocalRef(cls);
    if (needRelease) Util_ReleaseEnv();
}

void device_on_qr_code_scan(const char *username, const char *avatar, unsigned int scanTime)
{
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "device_on_qr_code_scan");
    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER,
                            "device_on_qr_code_refresh NULL == tx_service");
        return;
    }

    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env) return;

    jclass cls = env->GetObjectClass(tx_service);
    jmethodID methodID = env->GetMethodID(cls, "onQrCodeScan",
                                          "(Ljava/lang/String;Ljava/lang/String;I)V");
    if (methodID) {
        jstring usernameStr;
        ConvChar2JString(env, username, usernameStr);
        jstring avatarStr;
        ConvChar2JString(env, avatar, avatarStr);
        env->CallVoidMethod(tx_service, methodID, usernameStr, avatarStr, scanTime);
        env->DeleteLocalRef(usernameStr);
        env->DeleteLocalRef(avatarStr);
    }
    env->DeleteLocalRef(cls);
    if (needRelease) Util_ReleaseEnv();
}

void device_on_bind_ticket_callback(int errCode, const char *ticket, unsigned int expireTime, unsigned int expireIn)
{
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "device_on_bind_ticket_callback");
    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER,
                            "device_on_bind_ticket_callback NULL == tx_service");
        return;
    }

    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env) return;

    jclass cls = env->GetObjectClass(tx_service);
    jmethodID methodID = env->GetMethodID(cls, "onTicketRefresh",
                                          "(Ljava/lang/String;II)V");
    if (methodID) {
        jstring ticketStr;
        ConvChar2JString(env, ticket, ticketStr);
        env->CallVoidMethod(tx_service, methodID, ticketStr, expireTime, expireIn);
        env->DeleteLocalRef(ticketStr);
    }
    env->DeleteLocalRef(cls);
    if (needRelease) Util_ReleaseEnv();
}

JNIEXPORT jint JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_eraseAllBinders(JNIEnv *env, jclass) {
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "eraseAllBinders");
    return device_erase_all_binders();
}

JNIEXPORT jint JNICALL
Java_com_tencent_xiaowei_sdk_XWSDKJNI_eraseBinder(JNIEnv *env, jclass, jstring binderUserName) {

    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "eraseBinder");
    const char *pVal_binderUserName = NULL;
    if (binderUserName) {
        pVal_binderUserName = env->GetStringUTFChars(binderUserName, 0);
    }

    int nRet = device_erase_binders(pVal_binderUserName);

    if (pVal_binderUserName) {
        env->ReleaseStringUTFChars(binderUserName, pVal_binderUserName);
    }

    return nRet;
}

JNIEXPORT jobjectArray JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_fetchBinderList(JNIEnv *env, jclass) {
    jobjectArray arrayBinder = NULL;

    int nCount = 20;

    DEVICE_BINDER_INFO *pBinderList = new DEVICE_BINDER_INFO[20];
    if (pBinderList) {
        memset(pBinderList, 0, nCount * sizeof(DEVICE_BINDER_INFO));
        int err = device_get_binder_list(pBinderList, &nCount);
        __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "fetchBinderList, nCount=[%d]",nCount);
        if (error_null == err && nCount > 0) {
            arrayBinder = ConverBinderList2JobjectArray(env, pBinderList, nCount);
        }
        else if (error_memory_not_enough == err && nCount > 0 && nCount < 500 ) {
            DEVICE_BINDER_INFO *pBinderListBiger = new DEVICE_BINDER_INFO[nCount];
            if (pBinderListBiger) {
                memset(pBinderListBiger, 0, nCount * sizeof(DEVICE_BINDER_INFO));
                __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "fetchBinderList, last time memory is not enough, this time nCount=[%d]",nCount);
                int err = device_get_binder_list(pBinderListBiger, &nCount);
                if (error_null == err && nCount > 0) {
                    arrayBinder = ConverBinderList2JobjectArray(env, pBinderListBiger, nCount);
                }
            }
            delete[] pBinderListBiger;
                
        }
        delete[] pBinderList;
    }

    return arrayBinder;
}

JNIEXPORT jobjectArray JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_getUnverifyBinderList(JNIEnv *env, jclass) {
    jobjectArray arrayBinder = NULL;

    int nCount = 20;

    DEVICE_BINDER_INFO *pBinderList = new DEVICE_BINDER_INFO[20];
    if (pBinderList) {
        memset(pBinderList, 0, nCount * sizeof(DEVICE_BINDER_INFO));
        int err = device_get_all_unverify_bind_request(pBinderList, &nCount);
        __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "getUnverifyBinderList, nCount=[%d]",nCount);
        if (error_null == err && nCount > 0) {
            arrayBinder = ConverBinderList2JobjectArray(env, pBinderList, nCount);
        }
        else if (error_memory_not_enough == err && nCount > 0 && nCount < 500 ) {
            DEVICE_BINDER_INFO *pBinderListBiger = new DEVICE_BINDER_INFO[nCount];
            if (pBinderListBiger) {
                memset(pBinderListBiger, 0, nCount * sizeof(DEVICE_BINDER_INFO));
                __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "getUnverifyBinderList, last time memory is not enough, this time nCount=[%d]",nCount);
                int err = device_get_all_unverify_bind_request(pBinderListBiger, &nCount);
                if (error_null == err && nCount > 0) {
                    arrayBinder = ConverBinderList2JobjectArray(env, pBinderListBiger, nCount);
                }
            }
            delete[] pBinderListBiger;
                
        }
        delete[] pBinderList;
    }

    return arrayBinder;
}

JNIEXPORT jint JNICALL
Java_com_tencent_xiaowei_sdk_XWSDKJNI_handleBinderVerify(JNIEnv *env, jclass, jstring binderUserName, jboolean isAccept) {

    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "handleBinderVerify");
    const char *pVal_binderUserName = NULL;
    if (binderUserName) {
        pVal_binderUserName = env->GetStringUTFChars(binderUserName, 0);
    }

    int nRet = device_handle_binder_verify_request(pVal_binderUserName, (bool)isAccept);

    if (pVal_binderUserName) {
        env->ReleaseStringUTFChars(binderUserName, pVal_binderUserName);
    }

    return nRet;
}

JNIEXPORT jint JNICALL
Java_com_tencent_xiaowei_sdk_XWSDKJNI_updateBinderAlias(JNIEnv *env, jclass, jstring binderUserName, jstring alias) {

    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "updateBinderAlias");
    const char *pVal_binderUserName = NULL;
    if (binderUserName) {
        pVal_binderUserName = env->GetStringUTFChars(binderUserName, 0);
    }
     const char *pVal_alias = NULL;
    if (alias) {
        pVal_alias = env->GetStringUTFChars(alias, 0);
    }

    int nRet = device_update_binder_alias(pVal_binderUserName, pVal_alias);

    if (pVal_binderUserName) {
        env->ReleaseStringUTFChars(binderUserName, pVal_binderUserName);
    }
    if (pVal_alias) {
        env->ReleaseStringUTFChars(alias, pVal_alias);
    }

    return nRet;
}

#ifdef __cplusplus
}
#endif
