/*
 * Tencent is pleased to support the open source community by making  XiaoweiSDK Demo Codes available.
 *
 * Copyright (C) 2017 THL A29 Limited, a Tencent company. All rights reserved.
 *
 * Licensed under the MIT License (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://opensource.org/licenses/MIT
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 *
 */

#include "CommonDef.h"

#ifdef __cplusplus
extern "C" {
#endif

extern jobject tx_service;

/**
 * 登录结果
 * @param errcode
 */
void device_on_login_complete(int errcode)
{
	__android_log_print(ANDROID_LOG_INFO, LOGFILTER, "on_login_complete | code[%d]\n", errcode);

	if (tx_service)
	{
		bool needRelease = false;
		JNIEnv *env = Util_CreateEnv(&needRelease);
		if (!env)  return;

		jclass cls = env->GetObjectClass(tx_service);
		jmethodID methodID = env->GetMethodID(cls, "onLoginComplete", "(I)V");
		if (methodID)
		{
			env->CallVoidMethod(tx_service, methodID, errcode);
		}

		env->DeleteLocalRef(cls);
		if (needRelease) Util_ReleaseEnv();
	}
}

/**
 * 在线状态变化
 * @param newStatus
 */
void device_on_net_status_change(int newStatus) {
	__android_log_print(ANDROID_LOG_INFO, LOGFILTER, "on_online_status[%d]\n", newStatus);

	if (tx_service) {
		bool needRelease = false;
		JNIEnv *env = Util_CreateEnv(&needRelease);
		if (!env)
			return;

		jclass cls = env->GetObjectClass(tx_service);
		jmethodID methodID = env->GetMethodID(cls, "onNetStatusChange", "(I)V");

		if (methodID) {
			env->CallVoidMethod(tx_service, methodID, newStatus);
		}

		env->DeleteLocalRef(cls);

		if (needRelease) {
			Util_ReleaseEnv();
		}
	}
}

/**
 * 设备昵称发生变化，可能是在APP端改了之类的
 * @param nickname
 */
void device_on_nickname_update(int errorCode, const char *nickname, int nicknameLength)
{
	__android_log_print(ANDROID_LOG_INFO, LOGFILTER, "on_nickname_update[%s]\n", nickname);
	if (tx_service)
	{
		bool needRelease = false;
		JNIEnv *env = Util_CreateEnv(&needRelease);
		if (!env)
			return;

		jclass cls = env->GetObjectClass(tx_service);
		jmethodID methodID = env->GetMethodID(cls, "onNicNameChange",
											"(ILjava/lang/String;)V");
		if (methodID) {
			jstring nicknameStr;
			ConvChar2JString(env, nickname, nicknameStr);
			env->CallVoidMethod(tx_service, methodID, errorCode, nicknameStr);
			env->DeleteLocalRef(nicknameStr);
		}

		env->DeleteLocalRef(cls);

		if (needRelease) Util_ReleaseEnv();
	}
}

/**
 * 设备头像发生变化，可能是在APP端改了之类的
 * @param avatarurl
 */
void device_on_avatar_update(int errorCode, const char *avatarurl, int urlLength)
{
	__android_log_print(ANDROID_LOG_INFO, LOGFILTER, "device_on_avatar_update[%s]\n", avatarurl);
	if (tx_service)
	{
		bool needRelease = false;
		JNIEnv *env = Util_CreateEnv(&needRelease);
		if (!env)
			return;

		jclass cls = env->GetObjectClass(tx_service);
		jmethodID methodID = env->GetMethodID(cls, "onAvatarChange",
											"(ILjava/lang/String;)V");
		if (methodID) {
			jstring avatarurlStr;
			ConvChar2JString(env, avatarurl, avatarurlStr);
			env->CallVoidMethod(tx_service, methodID, errorCode, avatarurlStr);
			env->DeleteLocalRef(avatarurlStr);
		}

		env->DeleteLocalRef(cls);

		if (needRelease) Util_ReleaseEnv();
	}
}

void device_on_logout_complete(int errcode)
{
	__android_log_print(ANDROID_LOG_INFO, LOGFILTER, "device_on_logout_complete");

	if (tx_service) {
		bool needRelease = false;
		JNIEnv *env = Util_CreateEnv(&needRelease);
		if (!env)
			return;

		jclass cls = env->GetObjectClass(tx_service);
		jmethodID methodID = env->GetMethodID(cls, "onLogoutComplete", "(I)V");

		if (methodID) {
			env->CallVoidMethod(tx_service, methodID, errcode);
		}

		env->DeleteLocalRef(cls);

		if (needRelease) {
			Util_ReleaseEnv();
		}
	}
}

#ifdef __cplusplus
}
#endif
