/*
 * Tencent is pleased to support the open source community by making  XiaoweiSDK Demo Codes available.
 *
 * Copyright (C) 2017 THL A29 Limited, a Tencent company. All rights reserved.
 *
 * Licensed under the MIT License (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://opensource.org/licenses/MIT
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 *
 */
#include "CommonDef.h"

#ifdef JNI_VERSION_1_4
#define JNI_VER JNI_VERSION_1_4
#endif

#ifdef JNI_VERSION_1_5
#undef JNI_VER
#define JNI_VER JNI_VERSION_1_5
#endif
#ifdef JNI_VERSION_1_6
#undef JNI_VER
#define JNI_VER JNI_VERSION_1_6
#endif


#ifdef __cplusplus
extern "C" {
#endif

static JavaVM *g_JVM = NULL;

//全局
jobject tx_service = NULL;
jclass s_serviceClass = NULL;

//device层
jclass s_clsBinderInfo = NULL;
jclass s_clsAudioAccount = NULL;//第三方登录，尚未开通
jclass s_clsDeviceProfile = NULL;
jclass s_clsDeviceOTA = NULL;

//小微业务
jclass s_clsAudioContext = NULL;
jclass s_clsAudioRequestParam = NULL;
jclass s_clsAudioResource = NULL;
jclass s_clsAudioResGroup = NULL;
jclass s_clsAudioResponse = NULL;
jclass s_clsAudioState = NULL;
jclass s_clsXWLoginStatusInfo = NULL;//小微自己技能的登录态
jclass s_clsAudioTTSData = NULL;
jclass s_clsAudioAppInfo = NULL;
jclass s_clsLoaction = NULL;
jclass s_clsXWEventLogInfo = NULL;
jclass s_clsOnGetTicketListener = NULL;

//Voip业务
jclass s_clsVoipInfo = NULL;

int g_log_level = 0;

//JNI层固定代码begin
JNIEXPORT jint JNICALL JNI_OnLoad(JavaVM *vm, void *reserved) {
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "JNI_OnLoad");
    g_JVM = vm;
    return JNI_VERSION_1_4;
}

JNIEnv *Util_CreateEnv(bool *pNeedRelease) {
    if (pNeedRelease) *pNeedRelease = false;
    if (!g_JVM) {
        __android_log_write(ANDROID_LOG_ERROR, LOGFILTER, "JVM is NULL, no JVM yet");
        return NULL;
    }
    JNIEnv *env = NULL;
    if (JNI_OK != g_JVM->GetEnv(reinterpret_cast<void **> (&env), JNI_VER)) {
        if (JNI_OK == g_JVM->AttachCurrentThread(&env, NULL)) {
            if (pNeedRelease) *pNeedRelease = true;
        } else {
            __android_log_write(ANDROID_LOG_ERROR, LOGFILTER, "JVM could not create JNI env");
        }
    }
    return env;
}

void Util_ReleaseEnv() {
    if (g_JVM) {
        if (JNI_OK != g_JVM->DetachCurrentThread()) {
            __android_log_write(ANDROID_LOG_ERROR, LOGFILTER, "JVM could not release JNI env");
        }
    }
}


char checkUtfBytes(const char *bytes, const char **errorKind) {
    while (*bytes != '\0') {
        char utf8 = *(bytes++);
        // Switch on the high four bits.
        switch (utf8 >> 4) {
            case 0x00:
            case 0x01:
            case 0x02:
            case 0x03:
            case 0x04:
            case 0x05:
            case 0x06:
            case 0x07:
                // Bit pattern 0xxx. No need for any extra bytes.
                break;
            case 0x08:
            case 0x09:
            case 0x0a:
            case 0x0b:
            case 0x0f:
                /*
                 * Bit pattern 10xx or 1111, which are illegal start bytes.
                 * Note: 1111 is valid for normal UTF-8, but not the
                 * modified UTF-8 used here.
                 */
                *errorKind = "start";
                return utf8;
            case 0x0e:
                // Bit pattern 1110, so there are two additional bytes.
                utf8 = *(bytes++);
                if ((utf8 & 0xc0) != 0x80) {
                    *errorKind = "continuation";
                    return utf8;
                }
                // Fall through to take care of the final byte.
            case 0x0c:
            case 0x0d:
                // Bit pattern 110x, so there is one additional byte.
                utf8 = *(bytes++);
                if ((utf8 & 0xc0) != 0x80) {
                    *errorKind = "continuation";
                    return utf8;
                }
                break;
        }
    }
    return 0;
}

//JNI层固定代码end


int native_log_func(int level, const char *tag, const char *filename, int line, const char *funcname, const char *data)
{
    if (level < g_log_level) return 0;
    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env) return 0;

    if (s_serviceClass) {
        static jmethodID s_onGetSDKLog = NULL;
        if (s_onGetSDKLog == NULL) {
            s_onGetSDKLog = env->GetMethodID(s_serviceClass, "onGetSDKLog",
                                             "(ILjava/lang/String;Ljava/lang/String;ILjava/lang/String;Ljava/lang/String;)V");
        }

        if (s_onGetSDKLog) {
            jstring jTag;
            ConvChar2JString(env, tag, jTag);
            jstring jFileName;
            ConvChar2JString(env, filename, jFileName);
            jstring jData;
            ConvChar2JString(env, data, jData);
            jstring jFuncname;
            ConvChar2JString(env, funcname, jFuncname);
            env->CallVoidMethod(tx_service, s_onGetSDKLog, level, jTag, jFileName, line, jFuncname, jData);
            env->DeleteLocalRef(jTag);
            env->DeleteLocalRef(jFileName);
            env->DeleteLocalRef(jData);
            env->DeleteLocalRef(jFuncname);
        }
    }

    if (needRelease) Util_ReleaseEnv();
    return level > 0 ? 1:0;
}

//Device回调函数
extern void device_on_login_complete(int errcode);
extern void device_on_net_status_change(int newStatus);
extern void device_on_binder_list_change(int error, const DEVICE_BINDER_INFO *pBinderList, int nCount);
extern void device_on_logout_complete(int errcode);
extern void device_on_nickname_update(int errorCode, const char *nickname, int nicknameLength);
extern void device_on_avatar_update(int errorCode, const char *avatarurl, int urlLength);
extern void device_on_binder_verify(DEVICE_BINDER_INFO info);
extern void device_on_qr_code_refresh(int errCode, const char *qr_code_path, unsigned int expireTime, unsigned int expireIn);
extern void device_on_bind_ticket_callback(int errCode, const char *ticket, unsigned int expireTime, unsigned int expireIn);
extern void device_on_qr_code_scan(const char *username, const char *avatar, unsigned int scanTime);
// 查询OTA的回调 TODO
//extern void on_ota_info(int from, bool force, unsigned int version, const char *title,
//const char *desc, const char *url, const char *md5);

/**
 * 获得SDK版本
 * @param env
 * @return
 */
JNIEXPORT jintArray JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_getDeviceSDKVersion(JNIEnv *env, jclass) {
    jintArray arr = env->NewIntArray(4);

    unsigned int main_ver = 0;
    unsigned int sub_ver = 0;
    unsigned int fix_ver = 0;
    unsigned int build_no = 0;
    device_get_sdk_version(&main_ver, &sub_ver, &fix_ver, &build_no);

    __android_log_print(ANDROID_LOG_ERROR, LOGFILTER, "getSDKVersion: %d %d %d %d", main_ver, sub_ver,
                        fix_ver, build_no);

    jint vers[4] = {static_cast<jint>(main_ver), static_cast<jint>(sub_ver), static_cast<jint>(fix_ver), static_cast<jint>(build_no)};

    env->SetIntArrayRegion(arr, 0, 4, vers);

    return arr;
}

JNIEXPORT void JNICALL
Java_com_tencent_xiaowei_sdk_XWSDKJNI_initJNI(JNIEnv *env, jobject service, jint log_level) {
    g_log_level = log_level;

    if (tx_service == NULL) {
        tx_service = env->NewGlobalRef(service);
        jclass cls = env->GetObjectClass(tx_service);
        s_serviceClass = (jclass) env->NewGlobalRef(cls);

        //device
        jclass clsBinderInfo = env->FindClass("com/tencent/xiaowei/info/DeviceBinderInfo");
        s_clsBinderInfo = (jclass) env->NewGlobalRef(clsBinderInfo);
        jclass clsAudioAccount = env->FindClass("com/tencent/xiaowei/info/DeviceAccountInfo");
        s_clsAudioAccount = (jclass) env->NewGlobalRef(clsAudioAccount);
        jclass clsLoaction = env->FindClass("com/tencent/xiaowei/info/DeviceLocationInfo");
        s_clsLoaction = (jclass) env->NewGlobalRef(clsLoaction);
        jclass clsProfile = env->FindClass("com/tencent/xiaowei/info/DeviceProfile");
        s_clsDeviceProfile = (jclass) env->NewGlobalRef(clsProfile);
        jclass clsOTA = env->FindClass("com/tencent/xiaowei/info/DeviceOTAInfo");
        s_clsDeviceOTA = (jclass) env->NewGlobalRef(clsOTA);
        
        

        //xiaowei
        jclass clsAudioContext = env->FindClass("com/tencent/xiaowei/info/XWContextInfo");
        s_clsAudioContext = (jclass) env->NewGlobalRef(clsAudioContext);
        jclass clsAudioRequest = env->FindClass("com/tencent/xiaowei/info/XWRequestInfo");
        s_clsAudioRequestParam = (jclass) env->NewGlobalRef(clsAudioRequest);
        jclass clsAudioResource = env->FindClass("com/tencent/xiaowei/info/XWResourceInfo");
        s_clsAudioResource = (jclass) env->NewGlobalRef(clsAudioResource);
        jclass clsAudioResGroup = env->FindClass("com/tencent/xiaowei/info/XWResGroupInfo");
        s_clsAudioResGroup = (jclass) env->NewGlobalRef(clsAudioResGroup);
        jclass clsAudioResponse = env->FindClass("com/tencent/xiaowei/info/XWResponseInfo");
        s_clsAudioResponse = (jclass) env->NewGlobalRef(clsAudioResponse);
        jclass clsAudioState = env->FindClass("com/tencent/xiaowei/info/XWPlayStateInfo");
        s_clsAudioState = (jclass) env->NewGlobalRef(clsAudioState);
        jclass clsXWLoginStatusInfo = env->FindClass("com/tencent/xiaowei/info/XWLoginStatusInfo");
        s_clsXWLoginStatusInfo = (jclass) env->NewGlobalRef(clsXWLoginStatusInfo);
        jclass clsAudioTTSData = env->FindClass("com/tencent/xiaowei/info/XWTTSDataInfo");
        s_clsAudioTTSData = (jclass) env->NewGlobalRef(clsAudioTTSData);
        jclass clsAudioAppInfo = env->FindClass("com/tencent/xiaowei/info/XWAppInfo");
        s_clsAudioAppInfo = (jclass) env->NewGlobalRef(clsAudioAppInfo);
        jclass clsXWEventLogInfo = env->FindClass("com/tencent/xiaowei/info/XWEventLogInfo");
        s_clsXWEventLogInfo = (jclass) env->NewGlobalRef(clsXWEventLogInfo);
        jclass clsOnGetTicketListener = env->FindClass("com/tencent/xiaowei/sdk/DeviceSDK$CloudTicketListener");
        s_clsOnGetTicketListener = (jclass) env->NewGlobalRef(clsOnGetTicketListener);

        //voip
        jclass clsVoipInfo = env->FindClass("com/tencent/xiaowei/info/ILinkVoipInfo");
        s_clsVoipInfo = (jclass)env->NewGlobalRef(clsVoipInfo);
    }
}

/*
 * 初始化Device SDK，需要先初始化JNI之后使用
 */
JNIEXPORT jint JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_initDevice(JNIEnv *env, jobject service, 
         jstring strOsPlatform, jbyteArray license, jstring strSerialNum, jint iProductID, 
         jlong iProductVersion, jint iKeyVersion, jint iRunMode, jstring strSysPath, jlong ddwSysCapacity, jstring strAppid, jstring caPath) {
    if (NULL == strOsPlatform || NULL == license || NULL == strSerialNum || NULL == strSysPath ) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "Invalid Init Parmas");
        return error_param_invalid;
    }

    if (service == NULL) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER,
                            "The Android Service(DeviceSDK) hasn't been started");
        return error_failed;
    }

    if (tx_service == NULL) {
        Java_com_tencent_xiaowei_sdk_XWSDKJNI_initJNI(env, service, g_log_level);
    }

    const char *os_platform = env->GetStringUTFChars(strOsPlatform, 0);
    const char *serial_num = env->GetStringUTFChars(strSerialNum, 0);
    const char *sys_path = env->GetStringUTFChars(strSysPath, 0);
    const char *app_id = env->GetStringUTFChars(strAppid, 0);
    const char *ca_path = env->GetStringUTFChars(caPath, 0);

    int license_length = env->GetArrayLength(license);
    char *license_buf = new char[license_length + 1];
    memset(license_buf, 0, license_length + 1);
    if (license_buf) {
        env->GetByteArrayRegion(license, 0, license_length, (jbyte *) license_buf);
    }

    DEVICE_INFO info = {0};
    info.os_platform = os_platform;
    info.device_serial_number = (char *) serial_num;
    info.device_license = (char *) license_buf;
    info.product_id = iProductID;
    info.key_version = iKeyVersion;
    info.user_hardware_version = iProductVersion;
    info.run_mode = iRunMode;
    info.xiaowei_appid = app_id;
    info.ca_path = ca_path;

    DEVICE_NOTIFY notify = {0};
    notify.on_login_complete = device_on_login_complete;
    notify.on_net_status_changed = device_on_net_status_change;
    notify.on_binder_list_change = device_on_binder_list_change;
    notify.on_logout_complete = device_on_logout_complete;
    notify.on_nickname_update = device_on_nickname_update;
    notify.on_avatar_update = device_on_avatar_update;
    notify.on_binder_verify = device_on_binder_verify;
    notify.on_qr_code_refresh = device_on_qr_code_refresh;
    notify.on_qr_code_scan = device_on_qr_code_scan;

    DEVICE_INIT_PATH init_path = {0};
    init_path.system_path = (char *) sys_path;
    init_path.system_path_capacity = ddwSysCapacity;

    //先setLogFuc，后初始化
    device_set_log_func(native_log_func, g_log_level, false, true);


    //初始化设备
    int ret = device_init(&info, &notify, &init_path);
    if (error_null == ret) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "device_init success\n");

    } else {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "device_init failed [%d]\n", ret);
    }

    env->ReleaseStringUTFChars(strOsPlatform, os_platform);
    env->ReleaseStringUTFChars(strSerialNum, serial_num);
    env->ReleaseStringUTFChars(strSysPath, sys_path);
    env->ReleaseStringUTFChars(caPath, ca_path);

    if (license_buf) {
        delete[] license_buf;
    }
    return ret;
}

/**
 * 反初始化device SDK，此操作会同时反初始化当前所有的业务，如小微、VOIP等
 */
JNIEXPORT jint JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_unInitDevice(JNIEnv *env, jobject service)
{
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "unInitDevice");
    int ret = device_uninit();
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "unInitDevice, result is %d", ret);
    return ret;
}

/**
 * @brief Check the SDK status.
 * @retval 0 SDK is uninitialized
 * @retval 1 SDK initialization in progress
 * @retval 2 SDK initialize failed
 * @retval 3 SDK initialize success
 */
JNIEXPORT jint JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_checkStatus(JNIEnv *env, jclass cls)
{
    return 1;
}

/**
 * @brief Check the account status.
 * @retval 0 logout
 * @retval 1 logining
 * @retval 2 login
 * @retval 3 unregister
 */
JNIEXPORT jint JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_getAccountStatus(JNIEnv *env, jclass cls)
{
    return 1;
}

/**
 * 改设备自己的昵称
 *
 * @return 成功或者失败
 */

JNIEXPORT jint JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_updateDeviceNickname(JNIEnv *env, jclass, jstring nickname)
{
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "updateDeviceNickname");
    const char *pVal_nickname = NULL;
    if (nickname) {
        pVal_nickname = env->GetStringUTFChars(nickname, 0);
    }

    int nRet = device_update_device_nickname(pVal_nickname);

    if (pVal_nickname) {
        env->ReleaseStringUTFChars(nickname, pVal_nickname);
    }

    return nRet;
}

  

/**
 * 改设备自己的头像，请自己确保路径下的图片可用
 * @param jpgFilePath JPG格式的头像图片文件地址，注意是本文件，不是网络文件，512x512 或256x256
 *
 * @return 成功或者失败
 */
JNIEXPORT jint JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_updateDeviceAvatar(JNIEnv *env, jclass, jstring jpgFilePath)
{
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "updateDeviceAvatar");
    const char *pVal_jpgFilePath = NULL;
    if (jpgFilePath) {
        pVal_jpgFilePath = env->GetStringUTFChars(jpgFilePath, 0);
    }

    int nRet = device_update_device_avatar(pVal_jpgFilePath);

    if (pVal_jpgFilePath) {
        env->ReleaseStringUTFChars(jpgFilePath, pVal_jpgFilePath);
    }

    return nRet;
}

/**
 * @brief upLoad SDK log
 */
JNIEXPORT void JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_upLoadLog(JNIEnv *env, jclass cls)
{
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "upLoadLog");
    // device_upload_sdk_log();
}


/**
 * 主动刷新动态二维码，当二维码失效时应该刷新一下
 * @param env
 * @param cls
 * @return
 */
JNIEXPORT jint JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_refreshQRCodeUrl(JNIEnv *env, jclass cls) {

    return device_refresh_dynamic_qr_code();
}

/**
 * 获取ticket用于绑定
 * @param env
 * @param cls
 * @return
 */
JNIEXPORT jint JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_getBindTicket(JNIEnv *env, jclass cls) {
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER,
                        "getBindTicket jni");
    return device_get_bind_ticket(device_on_bind_ticket_callback);
}


/**
 * 获得设备uin
 * @param env
 * @param cls
 * @return
 */
JNIEXPORT jlong JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_getUin(JNIEnv *env, jclass cls) {
    return device_get_uin();
}

/**
 * 获得设备id
 * @param env
 * @param cls
 * @return
 */
JNIEXPORT jstring JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_getDeviceId(JNIEnv *env, jclass cls) {
    const char* device_id =  device_get_id();
    jstring str;
    ConvChar2JString(env, device_id , str);
    return str;
}

JNIEXPORT jlong JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_factoryReset(JNIEnv *env, jclass cls) {
    return device_unregister([](int errCode){});
}


/**
 * @GPS回调
 */
void m_on_gps_callback(XIAOWEI_GPS_INFO* gpsInfo) 
{
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER,
                        "on_gps_callback");

    if (NULL == gpsInfo) {
        return;
    }
    if (NULL == tx_service) {
        return;
    }

    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env) return;

    jclass cls = env->GetObjectClass(tx_service);

    jmethodID methodID = env->GetMethodID(cls, "onGpsCallback", "()Lcom/tencent/xiaowei/info/DeviceLocationInfo;");
    if (methodID) {
        jobject objLocation = env->CallObjectMethod(tx_service, methodID);
        jfieldID available_id = env->GetFieldID(s_clsLoaction, "isGpsAvailable", "Z");
        jfieldID longitude_id = env->GetFieldID(s_clsLoaction, "longitude", "D");
        jfieldID latitude_id = env->GetFieldID(s_clsLoaction, "latitude", "D");
        jfieldID altitude_id = env->GetFieldID(s_clsLoaction, "altitude", "D");
        jfieldID speed_id = env->GetFieldID(s_clsLoaction, "speed", "D");
        jfieldID utcTime_id = env->GetFieldID(s_clsLoaction, "utcTime", "J");

        gpsInfo->is_gps_available = env->GetBooleanField(objLocation,available_id);
        gpsInfo->longitude = env->GetDoubleField(objLocation,longitude_id);
        gpsInfo->latitude = env->GetDoubleField(objLocation,latitude_id);
        gpsInfo->altitude = env->GetDoubleField(objLocation,altitude_id);
        gpsInfo->utc_time = env->GetLongField(objLocation,utcTime_id);
        gpsInfo->speed = env->GetDoubleField(objLocation,speed_id);
        env->DeleteLocalRef(objLocation);
    } else {
        gpsInfo->is_gps_available = false;
    }

    env->DeleteLocalRef(cls);
    if (needRelease)
        Util_ReleaseEnv();
}


JNIEXPORT jint JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_setGpsCallback(JNIEnv *env, jclass) 
{
    return xiaowei_set_gps_callback(m_on_gps_callback);
}

JNIEXPORT jobject JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_getDeviceProfile(JNIEnv *env, jclass) {
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "getDeviceProfile");
    DEVICE_PROFILE profile = {0};
    int ret = device_get_device_profile(&profile);
    if (ret != 0) {
        __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "getDeviceProfile failed, ret = %d",ret);
        jobject obj = NULL;
        return obj;
    }
    jclass clsProfile = s_clsDeviceProfile;
    jfieldID defaultDeviceHead = env->GetFieldID(clsProfile, "defaultDeviceHead", "Ljava/lang/String;");
    jfieldID defaultDeviceName = env->GetFieldID(clsProfile, "defaultDeviceName", "Ljava/lang/String;");
    jfieldID DeviceHead = env->GetFieldID(clsProfile, "DeviceHead", "Ljava/lang/String;");
    jfieldID DeviceName = env->GetFieldID(clsProfile, "DeviceName", "Ljava/lang/String;");
    jmethodID init = env->GetMethodID(clsProfile, "<init>", "()V");

    jobject obj = env->NewObject(clsProfile, init);

    jstring j_defaultDeviceHead;
    ConvChar2JString(env, profile.default_device_name, j_defaultDeviceHead);
    env->SetObjectField(obj, defaultDeviceHead, j_defaultDeviceHead);
    jstring j_defaultDeviceName;
    ConvChar2JString(env, profile.default_device_head, j_defaultDeviceName);
    env->SetObjectField(obj, defaultDeviceName, j_defaultDeviceName);
    jstring j_DeviceHead;
    ConvChar2JString(env, profile.device_head, j_DeviceHead);
    env->SetObjectField(obj, DeviceHead, j_DeviceHead);
    jstring j_DeviceName;
    ConvChar2JString(env, profile.device_name, j_DeviceName);
    env->SetObjectField(obj, DeviceName, j_DeviceName);

    env->DeleteLocalRef(j_defaultDeviceHead);
    env->DeleteLocalRef(j_defaultDeviceName);
    env->DeleteLocalRef(j_DeviceHead);
    env->DeleteLocalRef(j_DeviceName);
    return obj;
}

JNIEXPORT jint JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_setCacheKey(JNIEnv *env, jclass cls, jstring k, jstring v)
{
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "setCacheKey");
    const char *pk = NULL;
    if (k) {
        pk = env->GetStringUTFChars(k, 0);
    }
    const char *pv = NULL;
    if (v) {
        pv = env->GetStringUTFChars(v, 0);
    }

    int nRet = device_upload_voice_link_result(pk, pv);

    if (pv) {
        env->ReleaseStringUTFChars(v, pv);
    }
    if (pk) {
        env->ReleaseStringUTFChars(k, pk);
    }

    return nRet;
}

#ifdef __cplusplus
}
#endif
