/*
 * Tencent is pleased to support the open source community by making  XiaoweiSDK Demo Codes available.
 *
 * Copyright (C) 2017 THL A29 Limited, a Tencent company. All rights reserved.
 *
 * Licensed under the MIT License (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://opensource.org/licenses/MIT
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 *
 */
#include "CommonDef.h"
#include "voipSDK.h"

#ifdef __cplusplus
extern "C" {
#endif

extern jobject tx_service;
extern jclass s_clsVoipInfo;


/**
 * 收到voip邀请
 * @param fromUsername 来自谁的邀请
 */
void voip_on_invited(const char *fromUsername, int type) {
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "voip_on_invited: fromUsername[%s],type=%d", fromUsername,type);
    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "voip_on_invited NULL == tx_service");
        return;
    }

    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env) return;

    jclass cls = env->GetObjectClass(tx_service);
    jmethodID methodID = env->GetMethodID(cls, "onVoipInvited", "(Ljava/lang/String;I)V");
    if (methodID) {
        jstring fromUsernameStr;
        ConvChar2JString(env, fromUsername, fromUsernameStr);
        env->CallVoidMethod(tx_service, methodID, fromUsernameStr, type);
        env->DeleteLocalRef(fromUsernameStr);
    }

    env->DeleteLocalRef(cls);

    if (needRelease) Util_ReleaseEnv();
}

/**
 * 加入房间结果
 * @param result 结果，0成功，非0失败
 * @param isMaster 是否主叫，true主叫，false被叫
 */
void voip_on_join_room(int result, bool isMaster) {
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "voip_on_join_room: result[%d], isMaster[%d]", result, isMaster);
    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "voip_on_join_room NULL == tx_service");
        return;
    }

    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env) return;

    jclass cls = env->GetObjectClass(tx_service);
    jmethodID methodID = env->GetMethodID(cls, "onVoipJoinRoom", "(IZ)V");
    
    if (methodID) {
        env->CallVoidMethod(tx_service, methodID, result, isMaster);
    }

    env->DeleteLocalRef(cls);

    if (needRelease) Util_ReleaseEnv();
}

/**
 * 主叫的时候邀请对方结果
 * @param result 结果，0成功，非0失败
 */
void voip_on_invite_result(int result) {
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "voip_on_invite_result: result[%d]", result);
    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "voip_on_invite_result NULL == tx_service");
        return;
    }

    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env) return;

    jclass cls = env->GetObjectClass(tx_service);
    jmethodID methodID = env->GetMethodID(cls, "onVoipInviteResult", "(I)V");
    
    if (methodID) {
        env->CallVoidMethod(tx_service, methodID, result);
    }

    env->DeleteLocalRef(cls);

    if (needRelease) Util_ReleaseEnv();
}

/**
 * voip取消，包括主被叫时对方挂断，以及被叫时加入一个非法的房间
 * @param isMaster 是否主叫，true主叫，false被叫
 */
void voip_on_cancel(bool isMaster)
{
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "voip_on_cancel: isMaster[%d]", isMaster);
    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "voip_on_cancel NULL == tx_service");
        return;
    }

    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env) return;

    jclass cls = env->GetObjectClass(tx_service);
    jmethodID methodID = env->GetMethodID(cls, "onVoipCancel", "(Z)V");
    
    if (methodID) {
        env->CallVoidMethod(tx_service, methodID, isMaster);
    }

    env->DeleteLocalRef(cls);

    if (needRelease) Util_ReleaseEnv();
}

/**
 * voip接通
 */
void voip_on_talking(void)
{
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "voip_on_talking");
    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "voip_on_talking NULL == tx_service");
        return;
    }

    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env) return;

    jclass cls = env->GetObjectClass(tx_service);
    jmethodID methodID = env->GetMethodID(cls, "onVoipTalking", "()V");
    
    if (methodID) {
        env->CallVoidMethod(tx_service, methodID);
    }

    env->DeleteLocalRef(cls);

    if (needRelease) Util_ReleaseEnv();
}

/**
 * voip结束
 */
void voip_on_finish(void)
{
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "voip_on_finish");
    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "voip_on_finish NULL == tx_service");
        return;
    }

    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env) return;

    jclass cls = env->GetObjectClass(tx_service);
    jmethodID methodID = env->GetMethodID(cls, "onVoipFinish", "()V");
    
    if (methodID) {
        env->CallVoidMethod(tx_service, methodID);
    }

    env->DeleteLocalRef(cls);

    if (needRelease) Util_ReleaseEnv();
}

/**
 * voip参数发生变化，客户端应该时刻遵循这个变化
 * @param sampleRate 采样率，一般16000
 * @param sampleLenInms 每个frame里面有多长时间数据，这个很关键，错了就没声音
 * @param extends 整个参数其实很多，这里是PB的序列化。可以先不管他
 */
void voip_on_param_change(unsigned int sampleRate, unsigned int sampleLenInms, unsigned int channels, const char *extends)
{
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "voip_on_param_change: sampleRate[%d],sampleLenInms[%d]", sampleRate,sampleLenInms);
    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "voip_on_param_change NULL == tx_service");
        return;
    }

    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env) return;

    jclass cls = env->GetObjectClass(tx_service);
    jmethodID methodID = env->GetMethodID(cls, "onVoipParamChange", "(IILjava/lang/String;)V");
    
    if (methodID) {
        jstring extendStr;
        ConvChar2JString(env, extends, extendStr);
        env->CallVoidMethod(tx_service, methodID, sampleRate,sampleLenInms,extendStr);
        env->DeleteLocalRef(extendStr);
    }

    env->DeleteLocalRef(cls);

    if (needRelease) Util_ReleaseEnv();
}

/**
 * @brief voip room member change
 * @param currentNumber size of memberIds
 * @param memberIds members
 */
void voip_on_member_change(unsigned int currentNumber, const int *memberIds, const char **memberUsernames) {
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "voip_on_member_change, currentNumber is %d",currentNumber);
    for (unsigned int i=0;i< currentNumber;i++) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER, 
        "voip_on_member_change, member[%d] id[%d] username[%s]",i,memberIds[i],memberUsernames[i]);
    }
    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "voip_on_member_change NULL == tx_service");
        return;
    }

    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env) return;

    jclass cls = env->GetObjectClass(tx_service);
    jmethodID methodID = env->GetMethodID(cls, "onVoipMemberChange", "([I[Ljava/lang/String;)V");
    
    if (methodID) {
        jintArray arr = NULL;
        jobjectArray strArr = NULL;
        if (currentNumber > 0) {
            arr = env->NewIntArray(currentNumber);
            env->SetIntArrayRegion(arr, 0, currentNumber, (const jint*)memberIds);
            strArr = env->NewObjectArray(currentNumber, env->FindClass("java/lang/String"), NULL);
            // for(unsigned int i=0;i<currentNumber;i++) {  
            //     jstring name = env->NewStringUTF(memberUsernames[i]);
            //     env->SetObjectArrayElement(strArr,i, name);
            //     env->DeleteLocalRef(name);
            // }  
        }
        env->CallVoidMethod(tx_service, methodID, arr, strArr);
        if (arr) env->DeleteLocalRef(arr);
        if (strArr) env->DeleteLocalRef(strArr);
    }

    env->DeleteLocalRef(cls);

    if (needRelease) Util_ReleaseEnv();
}

/**
* @brief voip av status change
* @param audioEnableNumber size of audioEnableIds
* @param videoEnableNumber size of videoEnableIds
*/
void voip_on_av_change
(unsigned int audioEnableNumber, const int *audioEnableIds, unsigned int videoEnableNumber, const int *videoEnableIds) {
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, 
    "voip_on_av_change, audioEnableNumber is %d, videoEnableNumber is %d",audioEnableNumber, videoEnableNumber);
    for (unsigned int i=0;i< audioEnableNumber;i++) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "audioEnableNumber[%d] id: %d",i,audioEnableIds[i]);
    }
    for (unsigned int i=0;i< videoEnableNumber;i++) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "videoEnableNumber[%d] id: %d",i,videoEnableIds[i]);
    }
    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "voip_on_av_change NULL == tx_service");
        return;
    }

    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env) return;

    jclass cls = env->GetObjectClass(tx_service);
    jmethodID methodID = env->GetMethodID(cls, "onVoipAVMemberChange", "([I[I)V");
    
    if (methodID) {
        jintArray arr1 = NULL;
        jintArray arr2 = NULL;
        if (audioEnableNumber > 0) {
            arr1 = env->NewIntArray(audioEnableNumber);
            env->SetIntArrayRegion(arr1, 0, audioEnableNumber, (const jint*)audioEnableIds);
        }
        if (videoEnableNumber > 0) {
            arr2 = env->NewIntArray(videoEnableNumber);
            env->SetIntArrayRegion(arr2, 0, videoEnableNumber, (const jint*)videoEnableIds);
        }
        env->CallVoidMethod(tx_service, methodID, arr1, arr2);
        if (arr1) env->DeleteLocalRef(arr1);
        if (arr2) env->DeleteLocalRef(arr2);
    }

    env->DeleteLocalRef(cls);

    if (needRelease) Util_ReleaseEnv();
}

/**
 * 收到微信消息
 */
void wechat_on_message(const char *fromUsername, const char *fromNickname, 
const char *fromHeadImageUrl, const char *content, int mediaType) {
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "wechat_on_message: fromUsername[%s]", fromUsername);
    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER,
                            "wechat_on_message NULL == tx_service");
        return;
    }

    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env) return;

    jclass cls = env->GetObjectClass(tx_service);
    jmethodID methodID = env->GetMethodID(cls, "onReceiveMessage",
                                          "(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;I)V");
    
    if (methodID) {
        jstring fromUsernameStr;
        ConvChar2JString(env, fromUsername, fromUsernameStr);
        jstring fromNicknameStr;
        ConvChar2JString(env, fromNickname, fromNicknameStr);
        jstring fromHeadImageUrlStr;
        ConvChar2JString(env, fromHeadImageUrl, fromHeadImageUrlStr);
        jstring contentStr;
        ConvChar2JString(env, content, contentStr);
        env->CallVoidMethod(tx_service, methodID, fromUsernameStr, 
        fromNicknameStr, fromHeadImageUrlStr, contentStr, mediaType);
        env->DeleteLocalRef(fromNicknameStr);
        env->DeleteLocalRef(fromUsernameStr);
        env->DeleteLocalRef(fromHeadImageUrlStr);
        env->DeleteLocalRef(contentStr);
    }

    env->DeleteLocalRef(cls);

    if (needRelease) Util_ReleaseEnv();
}

/**
 * 发送微信消息的结果
 */
void wechat_send_result(int errCode) {
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "wechat_send_result: errCode[%d]", errCode);
    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER,
                            "wechat_send_result NULL == tx_service");
        return;
    }

    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env) return;

    jclass cls = env->GetObjectClass(tx_service);
    jmethodID methodID = env->GetMethodID(cls, "onSendMessage", "(I)V");
    
    if (methodID) {
        env->CallVoidMethod(tx_service, methodID, errCode);
    }

    env->DeleteLocalRef(cls);

    if (needRelease) Util_ReleaseEnv();
}

/**
 * 主动发起通话，appid一般应该是空
 */
JNIEXPORT int Java_com_tencent_xiaowei_sdk_XWSDKJNI_createVoipCall(JNIEnv *env, jclass, jstring username, jstring appid, jint type) {
    
    const char *pVal_username = NULL;
    if (username) {
        pVal_username = env->GetStringUTFChars(username, 0);
    }
    const char *pVal_appid = NULL;
    if (appid) {
        pVal_appid = env->GetStringUTFChars(appid, 0);
    }
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "createVoipCall: username %s.appid:%s, type:%d", pVal_username,pVal_appid,type);
    int nRet = voip_create_call(pVal_username, pVal_appid, type);

    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "createVoipCall: ret %d.", nRet);

    if (pVal_username) {
        env->ReleaseStringUTFChars(username, pVal_username);
    }
    if (pVal_appid) {
         env->ReleaseStringUTFChars(appid, pVal_appid);
    }

    return nRet;
}

/**
 * 接电话
 */
JNIEXPORT int Java_com_tencent_xiaowei_sdk_XWSDKJNI_answerVoipCall(JNIEnv *env, jclass) {
    int nRet = voip_answer();
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "answerVoipCall: ret %d.", nRet);
    return nRet;
}

/**
 * 挂电话
 */
JNIEXPORT int Java_com_tencent_xiaowei_sdk_XWSDKJNI_hangUpVoipCall(JNIEnv *env, jclass) {
    int nRet = voip_hangup();
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "hangUpVoipCall: ret %d.", nRet);
    return nRet;
}

JNIEXPORT int Java_com_tencent_xiaowei_sdk_XWSDKJNI_SendAudioData(JNIEnv *env, jclass, jbyteArray data,jint dataLen,jint playDelayMS) {
    // __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "sendAudioData: dataLen[%d]. delay[%d]", dataLen, playDelayMS);

    if (dataLen < 1) return 1;

    char *audio_data = new char[dataLen];
    jbyte *buf = env->GetByteArrayElements(data, JNI_FALSE);
    memset(audio_data, 0, dataLen);
    memcpy(audio_data, buf, dataLen);
    
    int nRet = voip_send_audio_data((unsigned char*)audio_data, dataLen, playDelayMS);

    env->ReleaseByteArrayElements(data, buf, 0);
    delete[] audio_data;
    audio_data = NULL;

    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "sendAudioData: ret %d.", nRet);

    return nRet;
}

JNIEXPORT int Java_com_tencent_xiaowei_sdk_XWSDKJNI_GetAudioData(JNIEnv *env, jclass, jbyteArray data,jint len) {
    // __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "getAudioData: dataLen[%d]", len);

    if (len < 1) return 1;

    jbyte* jniBuf1 = (env)->GetByteArrayElements(data, JNI_FALSE);

    int nRet = voip_get_audio_data((unsigned char*)jniBuf1, len);

    (env)->ReleaseByteArrayElements(data, jniBuf1, 0);

    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "getAudioData: ret %d.", nRet);

    return nRet;
}

JNIEXPORT int Java_com_tencent_xiaowei_sdk_XWSDKJNI_SendVideoData(JNIEnv *env, jclass, jbyteArray data,jint len,jint w,jint h,jint f) {
    // __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "sendVideoData: dataLen[%d]. width[%d]. height[%d]. format[%d]", len, w, h, f);

    if (len < 1) return 1;

    char *video_data = new char[len];
    jbyte *buf = env->GetByteArrayElements(data, JNI_FALSE);
    memset(video_data, 0, len);
    memcpy(video_data, buf, len);
    
    int nRet = voip_send_video_data((unsigned char*)video_data, len, w, h, f);

    env->ReleaseByteArrayElements(data, buf, 0);
    delete[] video_data;
    video_data = NULL;

    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "SendVideoData: ret %d.", nRet);

    return nRet;
}

JNIEXPORT int Java_com_tencent_xiaowei_sdk_XWSDKJNI_GetVideoData(JNIEnv *env, jclass cls, jbyteArray data) {
    // __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "getVideoData");

    jbyte* jniBuf1 = (env)->GetByteArrayElements(data, JNI_FALSE);
    int outW= 0;
    int outH = 0;
    int outLen = 0;
    int outMember = -1;
    int outFormat = -1;

    int nRet = voip_get_video_data(&outMember, (unsigned char*)jniBuf1, &outLen, &outW, &outH, &outFormat);
    try {
        // jclsToJobj(env, cls);
        jfieldID id_remoteImgWidth = env->GetStaticFieldID(cls, "field_remoteImgWidth", "I");
        jfieldID id_remoteImgHeight = env->GetStaticFieldID(cls, "field_remoteImgHeight", "I");
        jfieldID id_remoteImgLength = env->GetStaticFieldID(cls, "field_remoteImgLength", "I");
        jfieldID id_remoteImgMember = env->GetStaticFieldID(cls, "field_remoteImgMember", "I");
        jfieldID id_remoteOutFormat = env->GetStaticFieldID(cls, "field_remoteOutFormat", "I");
        jfieldID id_remoteImgOrien = env->GetStaticFieldID(cls, "field_remoteImgOrien", "I");
        env->SetStaticIntField(cls, id_remoteImgWidth, outW);
        env->SetStaticIntField(cls, id_remoteImgHeight, outH);
        env->SetStaticIntField(cls, id_remoteImgLength, outLen);
        env->SetStaticIntField(cls, id_remoteImgMember, outMember);
        env->SetStaticIntField(cls, id_remoteOutFormat, outFormat);
        env->SetStaticIntField(cls, id_remoteImgOrien, nRet);
    } catch (...) {
        __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "getVideoData failed");
    }

    (env)->ReleaseByteArrayElements(data, jniBuf1, 0);

    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "getVideoData: ret %d.", nRet);

    return nRet;
}


JNIEXPORT int Java_com_tencent_xiaowei_sdk_XWSDKJNI_startVoipService(JNIEnv *env, jclass,
jint audioType, jint closeAV, jint videoRatio, jint defaultVideoLength, jint cpuCoreNum, jint cpuFreq, jint cpuArch)
{
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "startVoipService");
    VOIP_CALLBACK voipCallback = {0};
    voipCallback.on_voip_invited = voip_on_invited;
    voipCallback.on_voip_join_room = voip_on_join_room;
    voipCallback.on_voip_invite_result = voip_on_invite_result;
    voipCallback.on_voip_cancel = voip_on_cancel;
    voipCallback.on_voip_talking = voip_on_talking;
    voipCallback.on_voip_finish = voip_on_finish;
    voipCallback.on_voip_member_change = voip_on_member_change;
    voipCallback.on_voip_param_change = voip_on_param_change;
    voipCallback.on_voip_av_change = voip_on_av_change;

    VOIP_INIT_PARAM param = {0};
    param.audio_type = audioType;
    param.def_close_av = closeAV;
    param.video_ratio = videoRatio;
    param.def_video_length = defaultVideoLength;
    param.cpu_core_number = cpuCoreNum;
    param.cpu_freq = cpuFreq;
    param.cpu_arch = cpuArch;
    // unsigned int audio_type;           /// 0 silk，1 silk+aac-ld, only silk+aac-ld support for xiaowei
    // unsigned int def_close_av;         /// 1:close audio，2:close video, 0:default. for linux device, must be 2
    // unsigned int video_ratio;          /// 0:3/4，1:6/19，2:1/1，3:1/2
    // unsigned int def_video_length;     /// default length for long side, such as 640.(e.g. if video_ratio = 0, it will be 640:480)
    // /** device hardware info */
    // unsigned int cpu_core_number;      /// CPU core munber, must > 0
    // unsigned int cpu_freq;             /// CPU frequency（unit: MHz）
    // unsigned int cpu_arch;             /// CPU architecture（1: armv5，2: armv6，4: armv7）

    return voip_service_start(&voipCallback, &param);
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "startVoipService OK");
}

JNIEXPORT int Java_com_tencent_xiaowei_sdk_XWSDKJNI_stopVoipService(JNIEnv *env, jclass)
{
    return voip_service_stop();
}

JNIEXPORT int Java_com_tencent_xiaowei_sdk_XWSDKJNI_switchVoipAV(JNIEnv *env, jclass, jboolean is_open_audio, jboolean is_open_video)
{
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "switchVoipAV");
    return voip_switch_AV(is_open_audio, is_open_video);
}

JNIEXPORT int Java_com_tencent_xiaowei_sdk_XWSDKJNI_setVoipAppCmd(
    JNIEnv *env, jclass, jint type, jbyteArray parameter, jint len) {
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "setVoipAppCmd, type:%d", type);
    if (len < 1) {
      jbyte *buf;
      int nRet = voip_set_app_cmd(type, (unsigned char*)buf, len);
      __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "setVoipAppCmd: ret %d.", nRet);
      return nRet;
    }
    jbyte *buf = env->GetByteArrayElements(parameter, JNI_FALSE);
    int nRet = voip_set_app_cmd(type, (unsigned char*)buf, len);
    env->ReleaseByteArrayElements(parameter, buf, 0);
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "setVoipAppCmd: ret %d.", nRet);

    return nRet;
}

struct MVQQVideoFmt
{
	int  iWidth;
	int  iHeight;
	int  iFormat;
	int  iTsMS;
    int  iDstRGBFmt;    
};

JNIEXPORT int Java_com_tencent_xiaowei_sdk_XWSDKJNI_voipVideoPreEncProcessCloud(
    JNIEnv *env, jclass cls, jbyteArray recordData,jint len,jint videoWidth,jint videoHeight,jint format,jbyteArray encImg) {
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "voipVideoPreEncProcessCloud");

    jbyte* jniBuf1  = (env)->GetByteArrayElements(recordData,0);
    unsigned char* recordBuffer = (unsigned char*)jniBuf1;
    jbyte* jniBuf2  = (env)->GetByteArrayElements(encImg,0);
    unsigned char* encImgBuffer = (unsigned char*)jniBuf2;

    MVQQVideoFmt stSrcFmt, stDetFmt;
    stSrcFmt.iWidth = videoWidth;
    stSrcFmt.iHeight = videoHeight;
    stSrcFmt.iTsMS = 0;
    stSrcFmt.iFormat = format & 0x3F;

    memcpy(&stDetFmt, &stSrcFmt, sizeof(MVQQVideoFmt));

    // 这里无需lock，在c++层的实现已经lock了
    int nRet = voip_video_pre_enc_process_cloud(recordBuffer,  encImgBuffer, len, &stSrcFmt, &stDetFmt);

    env->ReleaseByteArrayElements(recordData, jniBuf1, 0);
    env->ReleaseByteArrayElements(encImg, jniBuf2, 0);

    try {
        jfieldID id_HWEncW = env->GetStaticFieldID(cls, "field_HWEncW", "I");
        jfieldID id_HWEncH = env->GetStaticFieldID(cls, "field_HWEncH", "I");
        env->SetStaticIntField(cls, id_HWEncW, stSrcFmt.iWidth);
        env->SetStaticIntField(cls, id_HWEncH, stSrcFmt.iHeight);
    } catch (...) {
        __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "voipVideoPreEncProcessCloud convert HWEnc failed");
    }

    return nRet;
}

JNIEXPORT int Java_com_tencent_xiaowei_sdk_XWSDKJNI_voipSetVideoResolution(
    JNIEnv *env, jclass cls, jbyteArray pb, jint pbLen) {
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "voipSetVideoResolution");
    jbyte* jniBuf  = (env)->GetByteArrayElements(pb,0);
    unsigned char* pbBuf = (unsigned char*)jniBuf;
    int nRet = voip_set_video_resolution(pbBuf, pbLen);
    env->ReleaseByteArrayElements(pb, jniBuf, 0);
    return nRet;
}


JNIEXPORT int Java_com_tencent_xiaowei_sdk_XWSDKJNI_getVoipVoiceActivity(JNIEnv *env, jclass, jint member_id)
{
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "getVoipVoiceActivity");
    return voip_get_voice_activity(member_id);
}

JNIEXPORT jintArray JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_getVoipVadMember(JNIEnv *env, jclass) {
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "getVoipVadMember");
    int memberList[300] = {0};
    int ret = voip_get_vad_members(memberList);
    if (ret == 0) {
        __android_log_print(ANDROID_LOG_ERROR, LOGFILTER, "getVoipVadMember: failed");
        jintArray aar = NULL;
        return aar;
    }
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "getVoipVadMember native call finish");
    jintArray arr = env->NewIntArray(ret);
    env->SetIntArrayRegion(arr, 0, ret, (const jint*)memberList);

    return arr;
}

#ifdef __cplusplus
}
#endif
