/*
 * Tencent is pleased to support the open source community by making  XiaoweiSDK Demo Codes available.
 *
 * Copyright (C) 2017 THL A29 Limited, a Tencent company. All rights reserved.
 *
 * Licensed under the MIT License (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://opensource.org/licenses/MIT
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 *
 */
#include "CommonDef.h"


#ifdef __cplusplus
extern "C" {
#endif

extern jobject tx_service;
extern jclass s_clsDeviceOTA;

/** callbacks */
void device_on_ota_check_result(int err_code, const char *business_id, bool new_version, OTA_UPDATE_INFO* update_info, OTA_PACKAGE_INFO* package_info)
{
	__android_log_print(ANDROID_LOG_INFO, LOGFILTER, "device_on_ota_check_result, err=%d,new_version=%d",err_code, new_version);

    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "on_request_callback NULL == tx_service");
        return;
    }

    if ((!update_info || !package_info) && new_version)
    {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "device_on_ota_check_result, new version, but null obj");
    }
    
	if (tx_service) {
		bool needRelease = false;
		JNIEnv *env = Util_CreateEnv(&needRelease);
		if (!env)
			return;

        jstring StrBusinessId;
        ConvChar2JString(env, business_id, StrBusinessId);

		jclass cls = env->GetObjectClass(tx_service); 
		jmethodID methodID = env->GetMethodID(cls, "onOtaCheckResult", "(ILjava/lang/String;ZLcom/tencent/xiaowei/info/DeviceOTAInfo;)V");

        jclass clsOTAInfo = s_clsDeviceOTA;
        jmethodID initOTAInfo = env->GetMethodID(clsOTAInfo, "<init>", "()V");
        jobject objOTA = env->NewObject(clsOTAInfo, initOTAInfo);
        if (new_version) {
            jfieldID id_updateTime = env->GetFieldID(clsOTAInfo, "updateTime", "J");
            jfieldID id_updateType = env->GetFieldID(clsOTAInfo, "updateType", "I");
            jfieldID id_version = env->GetFieldID(clsOTAInfo, "version", "J");
            jfieldID id_updateLevel = env->GetFieldID(clsOTAInfo, "updateLevel", "I");
            jfieldID id_devFlag = env->GetFieldID(clsOTAInfo, "devFlag", "I");
            jfieldID id_versionName = env->GetFieldID(clsOTAInfo, "versionName", "Ljava/lang/String;");
            jfieldID id_updateTips = env->GetFieldID(clsOTAInfo, "updateTips", "Ljava/lang/String;");
            jfieldID id_extraInfo = env->GetFieldID(clsOTAInfo, "extraInfo", "Ljava/lang/String;");
            jfieldID id_packSize = env->GetFieldID(clsOTAInfo, "packSize", "J");
            jfieldID id_packUrl = env->GetFieldID(clsOTAInfo, "packUrl", "Ljava/lang/String;");
            jfieldID id_packHashType = env->GetFieldID(clsOTAInfo, "packHashType", "I");
            jfieldID id_packHash = env->GetFieldID(clsOTAInfo, "packHash", "Ljava/lang/String;");
            
            jstring strVersionName;
            ConvChar2JString(env, update_info->version_name, strVersionName);
            jstring strUpdateTips;
            ConvChar2JString(env, update_info->update_tips, strUpdateTips);
            jstring strExtraInfo;
            ConvChar2JString(env, update_info->extra_info, strExtraInfo);
            jstring strPackUrl;
            ConvChar2JString(env, package_info->package_url, strPackUrl);
            jstring strPackHash;
            ConvChar2JString(env, package_info->package_hash, strPackHash);

            env->SetLongField(objOTA, id_updateTime, update_info->update_time);
            env->SetIntField(objOTA, id_updateType, update_info->update_type);
            env->SetLongField(objOTA, id_version, update_info->version);
            env->SetIntField(objOTA, id_updateLevel, update_info->update_level);
            env->SetIntField(objOTA, id_devFlag, update_info->dev_flag);
            env->SetObjectField(objOTA, id_versionName, strVersionName);
            env->SetObjectField(objOTA, id_updateTips, strUpdateTips);
            env->SetObjectField(objOTA, id_extraInfo, strExtraInfo);
            env->SetLongField(objOTA, id_packSize, package_info->package_size);
            env->SetObjectField(objOTA, id_packUrl, strPackUrl);
            env->SetIntField(objOTA, id_packHashType, package_info->package_hash_type);
            env->SetObjectField(objOTA, id_packHash, strPackHash);

            if (methodID) {
			    env->CallVoidMethod(tx_service, methodID, err_code, StrBusinessId, (jboolean)new_version, objOTA);
            }

            if (strVersionName) {
                env->DeleteLocalRef(strVersionName);
            }
            if (strUpdateTips) {
                env->DeleteLocalRef(strUpdateTips);
            }
            if (strExtraInfo) {
                env->DeleteLocalRef(strExtraInfo);
            }
            if (strPackUrl) {
                env->DeleteLocalRef(strPackUrl);
            }
            if (strPackHash) {
                env->DeleteLocalRef(strPackHash);
            }
		}
        else
        {
            if (methodID) {
			    env->CallVoidMethod(tx_service, methodID, err_code, StrBusinessId, (jboolean)new_version, objOTA);
            }
        }
        if (StrBusinessId) {
                env->DeleteLocalRef(StrBusinessId);
            }   
 
        if (objOTA) {
                env->DeleteLocalRef(objOTA);
        }

		env->DeleteLocalRef(cls);

		if (needRelease) {
			Util_ReleaseEnv();
		}
	}
}

/** download progress, for UI using*/
void device_on_ota_download_progress(const char *url, double current, double total)
{
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "device_on_ota_download_progress, url=%s,current=%f,total=%f",url, current,total);

    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "device_on_ota_download_progress NULL == tx_service");
        return;
    }

    if (tx_service) {
		bool needRelease = false;
		JNIEnv *env = Util_CreateEnv(&needRelease);
		if (!env)
			return;

        jstring StrUrl;
        ConvChar2JString(env, url, StrUrl);

        jclass cls = env->GetObjectClass(tx_service); 
        jmethodID methodID = env->GetMethodID(cls, "onOtaDownLoadProgress", "(Ljava/lang/String;DD)V");
        if (methodID) {
			    env->CallVoidMethod(tx_service, methodID, StrUrl, (jdouble)current, (jdouble)total);
            }

        if (StrUrl) env->DeleteLocalRef(StrUrl);

        env->DeleteLocalRef(cls);
        if (needRelease) Util_ReleaseEnv();
    }
}

/** error_code > 0 see CURL. else -1,hash error;-2 unknown hash type. */
void device_on_ota_download_error(const char *url, int err_code)
{
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "device_on_ota_download_error, url=%s,errcode=%d",url, err_code);

    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "device_on_ota_download_error NULL == tx_service");
        return;
    }

    if (tx_service) {
		bool needRelease = false;
		JNIEnv *env = Util_CreateEnv(&needRelease);
		if (!env)
			return;

        jstring StrUrl;
        ConvChar2JString(env, url, StrUrl);

        jclass cls = env->GetObjectClass(tx_service); 
        jmethodID methodID = env->GetMethodID(cls, "onOtaDownLoadError", "(Ljava/lang/String;I)V");
        if (methodID) {
			    env->CallVoidMethod(tx_service, methodID, StrUrl, err_code);
            }

        if (StrUrl) env->DeleteLocalRef(StrUrl);
        env->DeleteLocalRef(cls);
        if (needRelease) Util_ReleaseEnv();
    }
}

/** download complete*/
void device_on_ota_download_complete(const char *url, const char *file_path)
{
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "device_on_ota_download_complete, url=%s,path=%s",url, file_path);

    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "device_on_ota_download_complete NULL == tx_service");
        return;
    }

    if (tx_service) {
		bool needRelease = false;
		JNIEnv *env = Util_CreateEnv(&needRelease);
		if (!env)
			return;

        jstring StrUrl;
        ConvChar2JString(env, url, StrUrl);
        jstring StrFilepath;
        ConvChar2JString(env, file_path, StrFilepath);

        jclass cls = env->GetObjectClass(tx_service); 
        jmethodID methodID = env->GetMethodID(cls, "onOtaDownLoadComplete", "(Ljava/lang/String;Ljava/lang/String;)V");
        if (methodID) {
			    env->CallVoidMethod(tx_service, methodID, StrUrl, StrFilepath);
            }

        if (StrUrl) env->DeleteLocalRef(StrUrl);
        if (StrUrl) env->DeleteLocalRef(StrFilepath);

        env->DeleteLocalRef(cls);
        if (needRelease) Util_ReleaseEnv();
    }
}

/**
 * @brief init OTA module(call this after device init finish if you need OTA)
 * @param callback {@link DEVICE_OTA_RESULT}
 * @return {@link device_error_code}
*/
JNIEXPORT int JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_deviceConfigOta(JNIEnv *env, jclass){
    DEVICE_OTA_RESULT result = {0};
    result.on_ota_check_result = device_on_ota_check_result;
    result.on_ota_download_progress = device_on_ota_download_progress;
    result.on_ota_download_error = device_on_ota_download_error;
    result.on_ota_download_complete = device_on_ota_download_complete;
    return device_config_ota(&result);
}


/**
 * @brief query OTA update info, then get the result in DEVICE_OTA_RESULT.on_ota_check_result
 * @param business_id A unique ID for the update business, same ID in another query will cancel current query. Get the business_id in xiaowie website during the register progress.
 * @param channel Update channel, you can use multiple channels for different update strategy.
 * @param ota_type
          enum UpdateType {
              UNKNOWN = 0,
              SOFTWARE = 1,
              FIRMWARE = 2,
              HARDWARE = 3,
          };
 * @param debug_flag 0 = normal, others for debug
 *
 * @return {@link device_error_code}
*/
JNIEXPORT int JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_queryOtaUpdate(JNIEnv *env, jclass, jstring business_id, jint channel,jint ota_type, jint debug_flag) {
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "queryOtaUpdate");                 
    
    const char *pVal_business_id = NULL;
    if (business_id) {
        pVal_business_id = env->GetStringUTFChars(business_id, 0);
    }

    int nRet = device_query_ota_update(pVal_business_id, channel, ota_type, debug_flag);

    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "queryOtaUpdate: ret %d.", nRet);

    if (pVal_business_id) {
        env->ReleaseStringUTFChars(business_id, pVal_business_id);
    }
    return nRet;
}

/**
 * 
 * @param env
 * @return
 */
JNIEXPORT int JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_otaDwonload(JNIEnv *env, jclass, jstring url, jstring download_path, jint hash_type, jstring hash_value) {
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "otaDwonload");  
    
    const char *pVal_url = NULL;
    if (url) {
        pVal_url = env->GetStringUTFChars(url, 0);
    }
    const char *pVal_download_path = NULL;
    if (download_path) {
        pVal_download_path = env->GetStringUTFChars(download_path, 0);
    }
    const char *pVal_hash_value = NULL;
    if (hash_value) {
        pVal_hash_value = env->GetStringUTFChars(hash_value, 0);
    }

    int nRet = device_ota_download(pVal_url, pVal_download_path, hash_type, pVal_hash_value);

    if (pVal_url) {
        env->ReleaseStringUTFChars(url, pVal_url);
    }
    if (pVal_download_path) {
        env->ReleaseStringUTFChars(download_path, pVal_download_path);
    }
    if (pVal_hash_value) {
        env->ReleaseStringUTFChars(hash_value, pVal_hash_value);
    }

    return nRet;
}

JNIEXPORT int JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_stopOtaDownload(JNIEnv *env, jclass, jstring url) {
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "stopOtaDownload");                 
    
    const char *pVal_url = NULL;
    if (url) {
        pVal_url = env->GetStringUTFChars(url, 0);
    }

    int nRet = device_ota_stop_download(pVal_url);

    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "stopOtaDownload: ret %d.", nRet);

    if (pVal_url) {
        env->ReleaseStringUTFChars(url, pVal_url);
    }
    return nRet;
}

#ifdef __cplusplus
}
#endif

