/*
 * Tencent is pleased to support the open source community by making  XiaoweiSDK Demo Codes available.
 *
 * Copyright (C) 2017 THL A29 Limited, a Tencent company. All rights reserved.
 *
 * Licensed under the MIT License (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://opensource.org/licenses/MIT
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 *
 */
#include "CommonDef.h"
#include "xiaoweiSDK.h"
#include "xiaoweiDecoder.h"

#ifdef __cplusplus
extern "C" {
#endif

extern jobject tx_service;
extern jclass s_serviceClass;

extern jclass s_clsAudioAppInfo;
extern jclass s_clsAudioContext;
extern jclass s_clsAudioRequestParam;
extern jclass s_clsAudioResource;
extern jclass s_clsAudioResGroup;
extern jclass s_clsAudioResponse;
extern jclass s_clsAudioTTSData;
extern jclass s_clsAudioState;
extern jclass s_clsXWLoginStatusInfo;
extern jclass s_clsXWEventLogInfo;
extern jclass s_clsOnGetTicketListener;

extern void xiaowei_on_device_exit(void);
extern void xiaowei_on_device_reboot(void);

/**
 * 网络耗时
 * @param voice_id
 * @param time
 * @return
 */
bool xiaowei_on_net_delay_callback(const char *voice_id, unsigned long long time) {
    if (NULL == tx_service) {
        return false;
    }

    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env)
        return false;

    jboolean bHandled = false;
    jclass cls = s_serviceClass;
    jmethodID methodID = env->GetMethodID(cls, "onNetworkDelayCallback", "(Ljava/lang/String;J)Z");
    if (methodID) {
        jstring strVoiceID;
        ConvChar2JString(env, voice_id, strVoiceID);
        bHandled = env->CallBooleanMethod(tx_service, methodID, strVoiceID, (jlong) time);
        env->DeleteLocalRef(strVoiceID);
    }

    if (needRelease) {
        Util_ReleaseEnv();
    }

    return bHandled;
}

/**
 * 请求结果回来了
 * @param voice_id
 * @param event
 * @param state_info
 * @param extend_info
 * @param extend_info_len
 * @return
 */

bool xiaowei_on_request_callback(const char *voice_id, XIAOWEI_EVENT event, const char *state_info,
                         const char *extend_info, unsigned int extend_info_len) {
    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "on_request_callback NULL == tx_service");
        return false;
    }

    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env)
        return false;

    jboolean bHandled = false;
    static jclass cls = s_serviceClass;

    static jclass clsAudioResponse = s_clsAudioResponse;
    static jclass clsAudioAppInfo = s_clsAudioAppInfo;
    static jclass clsAudioContext = s_clsAudioContext;
    static jclass clsAudioResource = s_clsAudioResource;
    static jclass clsAudioResGroup = s_clsAudioResGroup;
    static jfieldID id_appInfo = env->GetFieldID(clsAudioResponse, "appInfo",
                                                    "Lcom/tencent/xiaowei/info/XWAppInfo;");
    static jfieldID id_lastAppInfo = env->GetFieldID(clsAudioResponse, "lastAppInfo",
                                                    "Lcom/tencent/xiaowei/info/XWAppInfo;");

    static jfieldID id_resultCode = env->GetFieldID(clsAudioResponse, "resultCode",
                                                        "I");
    static jfieldID id_voiceID = env->GetFieldID(clsAudioResponse, "voiceID",
                                                    "Ljava/lang/String;");
    static jfieldID id_context = env->GetFieldID(clsAudioResponse, "context",
                                                    "Lcom/tencent/xiaowei/info/XWContextInfo;");
    static jfieldID id_requestText = env->GetFieldID(clsAudioResponse, "requestText",
                                                        "Ljava/lang/String;");
    static jfieldID id_responseType = env->GetFieldID(clsAudioResponse, "responseType",
                                                        "I");
    static jfieldID id_responseData = env->GetFieldID(clsAudioResponse,
                                                        "responseData",
                                                        "Ljava/lang/String;");
    static jfieldID id_intentInfoForUser = env->GetFieldID(clsAudioResponse, "intentInfoForUser",
                                                        "Ljava/lang/String;");
    static jfieldID id_autoTestData = env->GetFieldID(clsAudioResponse,
                                                        "autoTestData",
                                                        "Ljava/lang/String;");
    static jfieldID id_responseText = env->GetFieldID(clsAudioResponse,
                                                        "responseText",
                                                        "Ljava/lang/String;");
    static jfieldID id_controlID = env->GetFieldID(clsAudioResponse,
                                                        "controlID",
                                                        "I");
    static jfieldID id_controlValue = env->GetFieldID(clsAudioResponse,
                                                        "controlValue",
                                                        "Ljava/lang/String;");
    static jfieldID id_resources = env->GetFieldID(clsAudioResponse, "resources",
                                                        "[Lcom/tencent/xiaowei/info/XWResGroupInfo;");
    static jfieldID id_hasMorePlaylist = env->GetFieldID(clsAudioResponse,
                                                            "hasMorePlaylist", "Z");
    static jfieldID id_hasMorePlaylistUp = env->GetFieldID(clsAudioResponse,
                                                            "hasMorePlaylistUp", "Z");
    static jfieldID id_hasHistoryPlaylist = env->GetFieldID(clsAudioResponse,
                                                            "hasHistoryPlaylist", "Z");


    static jfieldID id_isRecovery = env->GetFieldID(clsAudioResponse, "recoveryAble",
                                                        "Z");
    static jfieldID id_playBehavior = env->GetFieldID(clsAudioResponse,
                                                        "playBehavior", "I");
    static jfieldID id_resourceListType = env->GetFieldID(clsAudioResponse,
                                                        "resourceListType", "I");

    static jfieldID id_isNotify = env->GetFieldID(clsAudioResponse, "isNotify",
                                                        "Z");

    static jfieldID id_wakeupFlag = env->GetFieldID(clsAudioResponse, "wakeupFlag",
                                                        "I");

    static jfieldID id_skillname = env->GetFieldID(clsAudioAppInfo, "name",
                                                    "Ljava/lang/String;");
    static jfieldID id_skillID = env->GetFieldID(clsAudioAppInfo, "ID",
                                                    "Ljava/lang/String;");
    static jfieldID id_apptype = env->GetFieldID(clsAudioAppInfo, "type", "I");

    static jfieldID id_ctxID = env->GetFieldID(clsAudioContext, "ID",
                                                    "Ljava/lang/String;");
    static jfieldID id_ctxspeakTimeout = env->GetFieldID(clsAudioContext,
                                                            "speakTimeout", "I");
    static jfieldID id_ctxsilentTimeout = env->GetFieldID(clsAudioContext,
                                                            "silentTimeout", "I");

    static jfieldID id_resformat = env->GetFieldID(clsAudioResource, "format", "I");
    static jfieldID id_resOffset = env->GetFieldID(clsAudioResource, "offset", "I");
    static jfieldID id_resPlayCount = env->GetFieldID(clsAudioResource, "playCount", "I");
    static jfieldID id_resID = env->GetFieldID(clsAudioResource, "ID",
                                                    "Ljava/lang/String;");
    static jfieldID id_rescontent = env->GetFieldID(clsAudioResource, "content",
                                                        "Ljava/lang/String;");
    static jfieldID id_resextendInfo = env->GetFieldID(clsAudioResource, "extendInfo",
                                                            "Ljava/lang/String;");

    static jfieldID id_groupResources = env->GetFieldID(clsAudioResGroup, "resources", "[Lcom/tencent/xiaowei/info/XWResourceInfo;");


    static jmethodID methodID = env->GetMethodID(cls, "onRequestCallback",
                                                       "(Ljava/lang/String;ILcom/tencent/xiaowei/info/XWResponseInfo;[B)Z");
    if (xiaowei_event_on_tts != event) {
        if (methodID) {
            jobject objRsp = NULL;
            XIAOWEI_PARAM_RESPONSE *pRsp = (XIAOWEI_PARAM_RESPONSE *) state_info;
            if (pRsp) {
                // jfieldID id_responseDataByte = env->GetFieldID(clsAudioResponse, "responseDataByte", "[B");

                //creat obj
                static jmethodID init = env->GetMethodID(clsAudioResponse, "<init>", "()V");
                objRsp = env->NewObject(clsAudioResponse, init);

                //AppInfo
                static jmethodID initAppInfo = env->GetMethodID(clsAudioAppInfo, "<init>", "()V");
                jobject objApp = env->NewObject(clsAudioAppInfo, initAppInfo);

                jobject objLastApp = env->NewObject(clsAudioAppInfo, initAppInfo);

                jstring strSkillName;
                ConvChar2JString(env, pRsp->skill_info.name, strSkillName);
                jstring strSkillID;
                ConvChar2JString(env, pRsp->skill_info.id, strSkillID);

                env->SetObjectField(objApp, id_skillname, strSkillName);
                env->SetObjectField(objApp, id_skillID, strSkillID);
                env->SetIntField(objApp, id_apptype, pRsp->skill_info.type);

                jstring strSkillName2;
                ConvChar2JString(env, pRsp->last_skill_info.name, strSkillName2);
                jstring strSkillID2;
                ConvChar2JString(env, pRsp->last_skill_info.id, strSkillID2);

                env->SetObjectField(objLastApp, id_skillname, strSkillName2);
                env->SetObjectField(objLastApp, id_skillID, strSkillID2);
                env->SetIntField(objLastApp, id_apptype, pRsp->last_skill_info.type);

                env->SetObjectField(objRsp, id_appInfo, objApp);
                env->SetObjectField(objRsp, id_lastAppInfo, objLastApp);

                env->DeleteLocalRef(strSkillName);
                env->DeleteLocalRef(strSkillID);
                env->DeleteLocalRef(objApp);
                env->DeleteLocalRef(strSkillName2);
                env->DeleteLocalRef(strSkillID2);
                env->DeleteLocalRef(objLastApp);

                //result code
                env->SetIntField(objRsp, id_resultCode, pRsp->error_code);

                //voice id
                jstring strVoiceID;
                ConvChar2JString(env, pRsp->voice_id, strVoiceID);
                env->SetObjectField(objRsp, id_voiceID, strVoiceID);
                env->DeleteLocalRef(strVoiceID);

                //ContextInfo
                static jmethodID initContextInfo = env->GetMethodID(clsAudioContext, "<init>",
                                                                      "()V");
                jobject objContext = env->NewObject(clsAudioContext, initContextInfo);

                jstring strCtxID;
                ConvChar2JString(env, pRsp->context.id, strCtxID);

                env->SetObjectField(objContext, id_ctxID, strCtxID);
                env->SetIntField(objContext, id_ctxspeakTimeout,
                                          (jint) pRsp->context.speak_timeout);
                env->SetIntField(objContext, id_ctxsilentTimeout,
                                          (jint) pRsp->context.silent_timeout);

                env->SetObjectField(objRsp, id_context, objContext);
                env->DeleteLocalRef(strCtxID);
                env->DeleteLocalRef(objContext);

                //request_text
                jstring strRequestText;
                ConvChar2JString(env, pRsp->request_text, strRequestText);
                env->SetObjectField(objRsp, id_requestText, strRequestText);
                env->DeleteLocalRef(strRequestText);

                //responseType
                env->SetIntField(objRsp, id_responseType, pRsp->response_type);

                //response_data
                jstring strRspExtend;
                ConvChar2JString(env, pRsp->response_data, strRspExtend);
                env->SetObjectField(objRsp, id_responseData, strRspExtend);
                env->DeleteLocalRef(strRspExtend);

                
                //intent_info_for_user
                jstring strIntentInfoForUser;
                ConvChar2JString(env, pRsp->intent_info, strIntentInfoForUser);
                env->SetObjectField(objRsp, id_intentInfoForUser, strIntentInfoForUser);
                env->DeleteLocalRef(strIntentInfoForUser);
                //answer
                jstring strResponseText;
                ConvChar2JString(env, pRsp->response_text, strResponseText);
                env->SetObjectField(objRsp, id_responseText, strResponseText);
                env->DeleteLocalRef(strResponseText);

                //control
                jstring strControlValue;
                ConvChar2JString(env, pRsp->control_value, strControlValue);
                env->SetObjectField(objRsp, id_controlValue, strControlValue);
                env->DeleteLocalRef(strControlValue);
                
                env->SetIntField(objRsp, id_controlID, pRsp->control_id);

                //resources
                static jmethodID initResGroup = env->GetMethodID(clsAudioResGroup, "<init>", "()V");

                static jmethodID initResource = env->GetMethodID(clsAudioResource, "<init>",
                                                                   "()V");
                jobjectArray arrayGroup = env->NewObjectArray(pRsp->resource_groups_size,
                                                                       clsAudioResGroup, NULL);
                for (int i = 0; i < pRsp->resource_groups_size; i++) {
                    jobject objGroup = env->NewObject(clsAudioResGroup, initResGroup);
                    jobjectArray arrayRes = env->NewObjectArray(pRsp->resource_groups[i].resources_size,
                                                                         clsAudioResource, NULL);
                    for (int j = 0; j < pRsp->resource_groups[i].resources_size; ++j) {
                        jobject objRes = env->NewObject(clsAudioResource, initResource);

                        jstring strResID;
                        ConvChar2JString(env, pRsp->resource_groups[i].resources[j].id, strResID);
                        jstring strResContent;
                        ConvChar2JString(env, pRsp->resource_groups[i].resources[j].content, strResContent);
                        jstring strResExtend;
                        ConvChar2JString(env, pRsp->resource_groups[i].resources[j].extend_buffer, strResExtend);


                        env->SetIntField(objRes, id_resformat, pRsp->resource_groups[i].resources[j].format);
                        env->SetIntField(objRes, id_resOffset, pRsp->resource_groups[i].resources[j].offset);
                        env->SetIntField(objRes, id_resPlayCount, pRsp->resource_groups[i].resources[j].play_count);
                        env->SetObjectField(objRes, id_resID, strResID);
                        env->SetObjectField(objRes, id_rescontent, strResContent);
                        env->SetObjectField(objRes, id_resextendInfo, strResExtend);

                        env->SetObjectArrayElement(arrayRes, j, objRes);

                        env->DeleteLocalRef(strResID);
                        env->DeleteLocalRef(strResContent);
                        env->DeleteLocalRef(strResExtend);
                        env->DeleteLocalRef(objRes);
                    }
                    env->SetObjectField(objGroup, id_groupResources, arrayRes);
                    env->SetObjectArrayElement(arrayGroup, i, objGroup);
                    env->DeleteLocalRef(objGroup);
                    env->DeleteLocalRef(arrayRes);
                }

                env->SetObjectField(objRsp, id_resources, arrayGroup);
                env->DeleteLocalRef(arrayGroup);

                //has_more_playlist down
                env->SetBooleanField(objRsp, id_hasMorePlaylist,
                                              (jboolean) pRsp->has_more_playlist);
                
                //has_more_playlist up
                env->SetBooleanField(objRsp, id_hasMorePlaylistUp,
                                              (jboolean) pRsp->has_more_playlist_up);

                env->SetBooleanField(objRsp, id_hasHistoryPlaylist,
                                              (jboolean) pRsp->has_history_playlist);

                //is_recovery
                env->SetBooleanField(objRsp, id_isRecovery, (jboolean) pRsp->is_recovery);
                env->SetBooleanField(objRsp, id_isNotify, (jboolean) pRsp->is_notify);
                env->SetIntField(objRsp, id_wakeupFlag, (jint) pRsp->wakeup_flag);

                //play_behavior
                env->SetIntField(objRsp, id_playBehavior, pRsp->play_behavior);

                env->SetIntField(objRsp, id_resourceListType, pRsp->resource_list_type);
            }
            jbyteArray jextend = NULL;
            if (extend_info_len > 0) {
                jextend = env->NewByteArray(extend_info_len);
                env->SetByteArrayRegion(jextend, 0, extend_info_len,
                                                 (jbyte *) extend_info);
            }

            // jbyteArray bytesArr = env->NewByteArray(strlen(pRsp->responseData));
            // env->SetByteArrayRegion(bytesArr, 0, strlen(pRsp->responseData), reinterpret_cast<jbyte*>(Rsp->responseData));
            // env->SetObjectField(obj, id_responseDataByte, bytesArr);

            jstring strVoiceID;
            ConvChar2JString(env, voice_id, strVoiceID);
            bHandled = env->CallBooleanMethod(tx_service, methodID, strVoiceID, event,
                                                       objRsp, jextend);

            if (objRsp) {
                env->DeleteLocalRef(objRsp);
            }
            if (strVoiceID) {
                env->DeleteLocalRef(strVoiceID);
            }
            if (jextend) {
                env->DeleteLocalRef(jextend);
            }
        }
    } else if (xiaowei_event_on_tts == event) {
    }

    if (needRelease) {
      Util_ReleaseEnv();
    }

    return bHandled;
}

/**
 * 开启服务
 * @param env
 * @param device
 * @param accountData
 * @return
 */
JNIEXPORT jint JNICALL
Java_com_tencent_xiaowei_sdk_XWSDKJNI_startXiaoweiService(JNIEnv *env, jclass) {
    XIAOWEI_CALLBACK callback = {0};
    callback.on_request_callback = xiaowei_on_request_callback;
    callback.on_net_delay_callback = xiaowei_on_net_delay_callback;

    XIAOWEI_NOTIFY notify = {0};
    notify.on_device_reboot = xiaowei_on_device_reboot;
    notify.on_service_exit = xiaowei_on_device_exit;

    return xiaowei_service_start(&callback, &notify);
}


/**
 * 停止服务
 * @param env
 * @return
 */
JNIEXPORT jint JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_stopXiaoweiService(JNIEnv *env, jclass) {
    return xiaowei_service_stop();
}

/**
 * 请求
 * @param env
 * @param type
 * @param requestData
 * @param context
 * @return
 */
JNIEXPORT jstring JNICALL
Java_com_tencent_xiaowei_sdk_XWSDKJNI_request(JNIEnv *env, jclass, jint type,
                                                 jbyteArray requestData, jobject param) {
    if (NULL == s_clsAudioRequestParam) {
        __android_log_print(ANDROID_LOG_ERROR, LOGFILTER, "request NULL == s_clsAudioRequestParam");
        return 0;
    }

    XIAOWEI_PARAM_CONTEXT stContext = {0};

    static jclass clsAudioRequest = s_clsAudioRequestParam;
    static jfieldID id_ID = env->GetFieldID(clsAudioRequest, "contextId", "Ljava/lang/String;");
    static jfieldID id_speakTimeout = env->GetFieldID(clsAudioRequest, "speakTimeout", "I");
    static jfieldID id_silentTimeout = env->GetFieldID(clsAudioRequest, "silentTimeout", "I");
    static jfieldID id_voiceRequestBegin = env->GetFieldID(clsAudioRequest, "voiceRequestBegin", "Z");
    static jfieldID id_voiceRequestEnd = env->GetFieldID(clsAudioRequest, "voiceRequestEnd", "Z");
    static jfieldID id_ctxprofileType = env->GetFieldID(clsAudioRequest, "profileType", "I");
    static jfieldID id_ctxvoiceWakeupType = env->GetFieldID(clsAudioRequest, "voiceWakeupType", "I");
    static jfieldID id_voiceWakeupText = env->GetFieldID(clsAudioRequest, "voiceWakeupText", "Ljava/lang/String;");
    static jfieldID id_requestParam = env->GetFieldID(clsAudioRequest, "requestParam", "J");

    jstring ID;
    jstring wakeupText;
    if(param != NULL) {
        ID = (jstring) env->GetObjectField(param, id_ID);
        jint speakTimeout = (jint) env->GetIntField(param, id_speakTimeout);
        jint silentTimeout = (jint) env->GetIntField(param, id_silentTimeout);
        jboolean voiceRequestBegin = (jboolean) env->GetBooleanField(param, id_voiceRequestBegin);
        jboolean voiceRequestEnd = (jboolean) env->GetBooleanField(param, id_voiceRequestEnd);
        jint profileType = (jint) env->GetIntField(param, id_ctxprofileType);
        jint voiceWakeupType = (jint) env->GetIntField(param, id_ctxvoiceWakeupType);
        wakeupText = (jstring) env->GetObjectField(param, id_voiceWakeupText);
        jlong requestParam = (jlong) env->GetLongField(param, id_requestParam);

        if (NULL != ID) {
            stContext.id = env->GetStringUTFChars(ID, 0);
        }
        stContext.speak_timeout = speakTimeout;
        stContext.silent_timeout = silentTimeout;
        stContext.voice_request_begin = voiceRequestBegin;
        stContext.voice_request_end = voiceRequestEnd;
        stContext.wakeup_profile = XIAOWEI_WAKEUP_PROFILE(profileType);
        stContext.wakeup_type = XIAOWEI_WAKEUP_TYPE(voiceWakeupType);
        stContext.request_param = requestParam;

        if(wakeupText) {
            stContext.wakeup_word = env->GetStringUTFChars(wakeupText, 0);
        }
    }

    char *chat_data = NULL;
    unsigned int char_data_len = (NULL == requestData) ? 0 : env->GetArrayLength(requestData);
    if (char_data_len > 0) {
        jbyte *buf = env->GetByteArrayElements(requestData, JNI_FALSE);
        chat_data = new char[char_data_len];
        memset(chat_data, 0, (unsigned int) char_data_len);
        memcpy(chat_data, buf, (unsigned int) char_data_len);
        env->ReleaseByteArrayElements(requestData, buf, 0);
    }

    char pVoiceID[33];
    memset(pVoiceID, 0, 33);
    int nRet = xiaowei_request(pVoiceID, (XIAOWEI_CHAT_TYPE) type, chat_data, char_data_len, &stContext);

    if (char_data_len > 0 && chat_data) {
        delete[] chat_data;
        chat_data = NULL;
    }
    if (stContext.id) {
        env->ReleaseStringUTFChars(ID, stContext.id);
    }
    if (stContext.wakeup_word) {
        env->ReleaseStringUTFChars(wakeupText, stContext.wakeup_word);
    }

    jstring strVoiceID;
    ConvChar2JString(env, pVoiceID, strVoiceID);
    return strVoiceID;
}

/**
 * 取消请求
 * @param env
 * @param voiceID
 * @return
 */
JNIEXPORT jint JNICALL
Java_com_tencent_xiaowei_sdk_XWSDKJNI_cancelRequest(JNIEnv *env, jclass, jstring voiceID) {
    const char *pstrVoiceID = (NULL == voiceID) ? NULL : env->GetStringUTFChars(voiceID, 0);

    int nRet = xiaowei_request_cancel(pstrVoiceID);

    if (pstrVoiceID) {
        env->ReleaseStringUTFChars(voiceID, pstrVoiceID);
    }

    return nRet;
}

/**
 * 取消TTS
 * @param env
 * @param voiceID
 * @return
 */
//这个接口多半要取消了，先暂时留在这里
JNIEXPORT jint JNICALL
Java_com_tencent_xiaowei_sdk_XWSDKJNI_cancelTTS(JNIEnv *env, jclass, jstring voiceID) {
    const char *pstrVoiceID = (NULL == voiceID) ? NULL : env->GetStringUTFChars(voiceID, 0);

    //int nRet = txca_tts_cancel(pstrVoiceID);
    int nRet = 0;

    if (pstrVoiceID) {
        env->ReleaseStringUTFChars(voiceID, pstrVoiceID);
    }

    return nRet;
}

/**
 * 上报音量变化
 * @param env
 * @param 当前音量
 * @return
 */
JNIEXPORT jint JNICALL
Java_com_tencent_xiaowei_sdk_XWSDKJNI_reportVolumeChange(JNIEnv *env, jclass, jint volume) {
    return xiaowei_report_volume(volume);
}

/**
 * 上报播放状态
 * @param env
 * @param state
 * @return
 */
JNIEXPORT jint JNICALL
Java_com_tencent_xiaowei_sdk_XWSDKJNI_reportPlayState(JNIEnv *env, jclass, jobject state) {
    if (NULL == s_clsAudioState ) {
        __android_log_print(ANDROID_LOG_ERROR, LOGFILTER,
                            "reportPlayState NULL == s_clsAudioState ");
        return 0;
    }

    if (NULL == state) {
        __android_log_print(ANDROID_LOG_ERROR, LOGFILTER, "reportPlayState NULL == state");
        return 0;
    }

    static jclass clsAudioState = s_clsAudioState;
    static jfieldID id_appInfo = env->GetFieldID(clsAudioState, "appInfo",
                                          "Lcom/tencent/xiaowei/info/XWAppInfo;");
    static jfieldID id_state = env->GetFieldID(clsAudioState, "state", "I");
    static jfieldID id_availability = env->GetFieldID(clsAudioState, "availability", "I");
    static jfieldID id_playID = env->GetFieldID(clsAudioState, "playID", "Ljava/lang/String;");
    static jfieldID id_playContent = env->GetFieldID(clsAudioState, "playContent", "Ljava/lang/String;");
    static jfieldID id_playOffset = env->GetFieldID(clsAudioState, "playOffset", "J");
    static jfieldID id_playMode = env->GetFieldID(clsAudioState, "playMode", "I");
    static jfieldID id_resourceType = env->GetFieldID(clsAudioState, "resourceType", "I");
    static jfieldID id_resourceName = env->GetFieldID(clsAudioState, "resourceName", "Ljava/lang/String;");
    static jfieldID id_performer = env->GetFieldID(clsAudioState, "performer", "Ljava/lang/String;");
    static jfieldID id_collection = env->GetFieldID(clsAudioState, "collection", "Ljava/lang/String;");
    static jfieldID id_coverUrl = env->GetFieldID(clsAudioState, "coverUrl", "Ljava/lang/String;");
    static jfieldID id_volume = env->GetFieldID(clsAudioState, "volume", "I");
    static jfieldID id_brightness = env->GetFieldID(clsAudioState, "brightness", "I");
    static jfieldID id_play_speed = env->GetFieldID(clsAudioState, "playSpeed", "F");

    jint j_state = (jint) env->GetIntField(state, id_state);
    jint j_availability = (jint) env->GetIntField(state, id_availability);
    jstring j_playID = (jstring) env->GetObjectField(state, id_playID);
    jstring j_playContent = (jstring) env->GetObjectField(state, id_playContent);
    jlong j_playOffset = (jlong) env->GetLongField(state, id_playOffset);
    jint j_playMode = (jint) env->GetIntField(state, id_playMode);
    jint j_resourceType = (jint) env->GetIntField(state, id_resourceType);
    jstring j_resourceName = (jstring) env->GetObjectField(state, id_resourceName);
    jstring j_performer = (jstring) env->GetObjectField(state, id_performer);
    jstring j_collection = (jstring) env->GetObjectField(state, id_collection);
    jstring j_coverUrl = (jstring) env->GetObjectField(state, id_coverUrl);
    jint j_volume = (jint) env->GetIntField(state, id_volume);
    jint j_brightness = (jint) env->GetIntField(state, id_brightness);
    jfloat j_play_speed = (jfloat) env->GetFloatField(state, id_play_speed);

    jclass clsAudioAppInfo = s_clsAudioAppInfo;
    jfieldID id_skillName = env->GetFieldID(clsAudioAppInfo, "name", "Ljava/lang/String;");
    jfieldID id_skillID = env->GetFieldID(clsAudioAppInfo, "ID", "Ljava/lang/String;");
    jfieldID id_appType = env->GetFieldID(clsAudioAppInfo, "type", "I");

    jobject appInfo = (jobject) env->GetObjectField(state, id_appInfo);
    jstring skillName = NULL;
    jstring skillID = NULL;
    jint appType = 0;
    if (appInfo) {
        skillName = (jstring) env->GetObjectField(appInfo, id_skillName);
        skillID = (jstring) env->GetObjectField(appInfo, id_skillID);
        appType = (jint) env->GetIntField(appInfo, id_appType);
    } 

    const char *pVal_skillName = NULL;
    if (skillName) {
        pVal_skillName = env->GetStringUTFChars(skillName, 0);
    }
    const char *pVal_skillID = NULL;
    if (skillID) {
        pVal_skillID = env->GetStringUTFChars(skillID, 0);
    }
    const char *pVal_playID = NULL;
    if (j_playID) {
        pVal_playID = env->GetStringUTFChars(j_playID, 0);
    }
    const char *pVal_playContent = NULL;
    if (j_playContent) {
        pVal_playContent = env->GetStringUTFChars(j_playContent, 0);
    }
    const char *pVal_resourceName = NULL;
    if (j_resourceName) {
        pVal_resourceName = env->GetStringUTFChars(j_resourceName, 0);
    }
    const char *pVal_performer = NULL;
    if (j_performer) {
        pVal_performer = env->GetStringUTFChars(j_performer, 0);
    }
    const char *pVal_collection = NULL;
    if (j_collection) {
        pVal_collection = env->GetStringUTFChars(j_collection, 0);
    }
    const char *pVal_coverUrl = NULL;
    if (j_coverUrl) {
        pVal_coverUrl = env->GetStringUTFChars(j_coverUrl, 0);
    }

    XIAOWEI_PARAM_STATE st = {0};
    st.skill_info.name = pVal_skillName;
    st.skill_info.id = pVal_skillID;
    st.skill_info.type = appType;
    st.play_state = (XIAOWEI_PLAY_STATE)j_state;
    // st.availability = (XIAOWEI_AVAILABILITY)j_availability;
    st.play_id = pVal_playID;
    st.play_content = pVal_playContent;
    st.play_offset = j_playOffset;
    st.play_mode = (XIAOWEI_PLAY_MODE)j_playMode;
    st.resource_type = (XIAOWEI_RESOURCE_FORMAT)j_playMode;
    st.resource_name = pVal_resourceName;
    st.performer = pVal_performer;
    st.collection = pVal_collection;
    st.cover_url = pVal_coverUrl;
    st.volume = j_volume;
    st.brightness = j_brightness;
    st.play_speed = j_play_speed;

    int nRet = xiaowei_report_state(&st);

    if (pVal_skillName) {
        env->ReleaseStringUTFChars(skillName, pVal_skillName);
    }
    if (pVal_skillID) {
        env->ReleaseStringUTFChars(skillID, pVal_skillID);
    }
    if (pVal_playID) {
        env->ReleaseStringUTFChars(j_playID, pVal_playID);
    }
    if (pVal_playContent) {
        env->ReleaseStringUTFChars(j_playContent, pVal_playContent);
    }
    if (pVal_resourceName) {
        env->ReleaseStringUTFChars(j_resourceName, pVal_resourceName);
    }
    if (pVal_performer) {
        env->ReleaseStringUTFChars(j_performer, pVal_performer);
    }
    if (pVal_collection) {
        env->ReleaseStringUTFChars(j_collection, pVal_collection);
    }
    if (pVal_coverUrl) {
        env->ReleaseStringUTFChars(j_coverUrl, pVal_coverUrl);
    }

    return nRet;
}


/**
 * 查询更多播放列表
 * @param env
 * @param skillId
 * @param playID
 * @param maxListSize
 * @param isUp
 * @return
 */
JNIEXPORT jstring JNICALL
Java_com_tencent_xiaowei_sdk_XWSDKJNI_getMorePlaylist(JNIEnv *env, jclass, jobject appInfo,
                                                     jstring playID, jint maxListSize,
                                                     jboolean isUp) {
    if (NULL == s_clsAudioAppInfo) {
        __android_log_print(ANDROID_LOG_ERROR, LOGFILTER,
                            "getMorePlaylist NULL == s_clsAudioAppInfo");
        return 0;
    }

    if (NULL == appInfo) {
        __android_log_print(ANDROID_LOG_ERROR, LOGFILTER, "getMorePlaylist NULL == appInfo");
        return 0;
    }

    jclass clsAudioAppInfo = s_clsAudioAppInfo;
    jfieldID id_skillName = env->GetFieldID(clsAudioAppInfo, "name", "Ljava/lang/String;");
    jfieldID id_skillID = env->GetFieldID(clsAudioAppInfo, "ID", "Ljava/lang/String;");
    jfieldID id_appType = env->GetFieldID(clsAudioAppInfo, "type", "I");

    jstring skillName = (jstring) env->GetObjectField(appInfo, id_skillName);
    jstring skillID = (jstring) env->GetObjectField(appInfo, id_skillID);
    jint appType = (jint) env->GetIntField(appInfo, id_appType);

    const char *pVal_skillName = NULL;
    if (skillName) {
        pVal_skillName = env->GetStringUTFChars(skillName, 0);
    }
    const char *pVal_skillID = NULL;
    if (skillID) {
        pVal_skillID = env->GetStringUTFChars(skillID, 0);
    }

    XIAOWEI_PARAM_SKILL st = {0};
    st.name = pVal_skillName;
    st.id = pVal_skillID;
    st.type = appType;

    const char *pPlayID = (NULL == playID) ? NULL : env->GetStringUTFChars(playID, 0);

    char pVoiceID[33];
    memset(pVoiceID, 0, 33);
    int nRet = 0;

    if (pVal_skillName) {
        env->ReleaseStringUTFChars(skillName, pVal_skillName);
    }
    if (pVal_skillID) {
        env->ReleaseStringUTFChars(skillID, pVal_skillID);
    }
    if (pPlayID) {
        env->ReleaseStringUTFChars(playID, pPlayID);
    }

    jstring strVoiceID;
    ConvChar2JString(env, pVoiceID, strVoiceID);
    return strVoiceID;
}
/**
 * 查询播放详情
 * @param env
 * @param appInfo
 * @param listPlayID
 * @return
 */
JNIEXPORT jstring JNICALL
Java_com_tencent_xiaowei_sdk_XWSDKJNI_getPlayDetailInfo(JNIEnv *env, jclass, jobject appInfo,
                                                           jobjectArray listPlayID) {
    if (NULL == s_clsAudioAppInfo) {
        __android_log_print(ANDROID_LOG_ERROR, LOGFILTER,
                            "getPlayDetailInfo NULL == s_clsAudioAppInfo");
        return 0;
    }

    if (NULL == appInfo) {
        __android_log_print(ANDROID_LOG_ERROR, LOGFILTER, "getPlayDetailInfo NULL == appInfo");
        return 0;
    }

    jclass clsAudioAppInfo = s_clsAudioAppInfo;
    jfieldID id_skillName = env->GetFieldID(clsAudioAppInfo, "name", "Ljava/lang/String;");
    jfieldID id_skillID = env->GetFieldID(clsAudioAppInfo, "ID", "Ljava/lang/String;");
    jfieldID id_appType = env->GetFieldID(clsAudioAppInfo, "type", "I");

    jstring skillName = (jstring) env->GetObjectField(appInfo, id_skillName);
    jstring skillID = (jstring) env->GetObjectField(appInfo, id_skillID);
    jint appType = (jint) env->GetIntField(appInfo, id_appType);

    const char *pVal_skillName = NULL;
    if (skillName) {
        pVal_skillName = env->GetStringUTFChars(skillName, 0);
    }
    const char *pVal_skillID = NULL;
    if (skillID) {
        pVal_skillID = env->GetStringUTFChars(skillID, 0);
    }

    XIAOWEI_PARAM_SKILL st = {0};
    st.name = pVal_skillName;
    st.id = pVal_skillID;
    st.type = appType;

    char **list_play_id = NULL;
    int list_size = (NULL == listPlayID) ? 0 : env->GetArrayLength(listPlayID);
    if (list_size > 0) {
        list_play_id = (char **) malloc(sizeof(char *) * list_size);
        memset(list_play_id, 0, (unsigned int) list_size * sizeof(char *));
        for (int i = 0; i < list_size; ++i) {
            jstring playidstr = (jstring) env->GetObjectArrayElement(listPlayID, i);
            if (playidstr != NULL) {
                list_play_id[i] = (char *) env->GetStringUTFChars(playidstr, 0);
            }
        }
    }

    char pVoiceID[33];
    memset(pVoiceID, 0, 33);
    int nRet =0;

    if (pVal_skillName) {
        env->ReleaseStringUTFChars(skillName, pVal_skillName);
    }
    if (pVal_skillID) {
        env->ReleaseStringUTFChars(skillID, pVal_skillID);
    }

    if (list_size > 0 && list_play_id) {
        for (int i = 0; i < list_size; ++i) {
            jstring playidstr = (jstring) env->GetObjectArrayElement(listPlayID, i);
            env->ReleaseStringUTFChars(playidstr, list_play_id[i]);
        }
        free(list_play_id);
    }

    jstring strVoiceID;
    ConvChar2JString(env, pVoiceID, strVoiceID);
    return strVoiceID;
}

/**
 * 刷新列表
 * @param env
 * @param appInfo
 * @param listPlayID
 * @return
 */
JNIEXPORT jstring JNICALL
Java_com_tencent_xiaowei_sdk_XWSDKJNI_refreshPlayList(JNIEnv *env, jclass, jobject appInfo,
                                                         jobjectArray listPlayID) {
    if (NULL == s_clsAudioAppInfo) {
        __android_log_print(ANDROID_LOG_ERROR, LOGFILTER,
                            "refreshPlayList NULL == s_clsAudioAppInfo");
        return 0;
    }

    if (NULL == appInfo) {
        __android_log_print(ANDROID_LOG_ERROR, LOGFILTER, "refreshPlayList NULL == appInfo");
        return 0;
    }
    
    jclass clsAudioAppInfo = s_clsAudioAppInfo;
    jfieldID id_skillName = env->GetFieldID(clsAudioAppInfo, "name", "Ljava/lang/String;");
    jfieldID id_skillID = env->GetFieldID(clsAudioAppInfo, "ID", "Ljava/lang/String;");
    jfieldID id_appType = env->GetFieldID(clsAudioAppInfo, "type", "I");

    jstring skillName = (jstring) env->GetObjectField(appInfo, id_skillName);
    jstring skillID = (jstring) env->GetObjectField(appInfo, id_skillID);
    jint appType = (jint) env->GetIntField(appInfo, id_appType);

    const char *pVal_skillName = NULL;
    if (skillName) {
        pVal_skillName = env->GetStringUTFChars(skillName, 0);
    }
    const char *pVal_skillID = NULL;
    if (skillID) {
        pVal_skillID = env->GetStringUTFChars(skillID, 0);
    }

    XIAOWEI_PARAM_SKILL st = {0};
    st.name = pVal_skillName;
    st.id = pVal_skillID;
    st.type = appType;

    char **list_play_id = NULL;
    int list_size = (NULL == listPlayID) ? 0 : env->GetArrayLength(listPlayID);
    if (list_size > 0) {
        list_play_id = (char **) malloc(sizeof(char *) * list_size);
        memset(list_play_id, 0, (unsigned int) list_size * sizeof(char *));
        for (int i = 0; i < list_size; ++i) {
            jstring playidstr = (jstring) env->GetObjectArrayElement(listPlayID, i);
            if (playidstr != NULL) {
                list_play_id[i] = (char *) env->GetStringUTFChars(playidstr, 0);
            }
        }
    }
    else
    {
        return 0;
    }
    

    char pVoiceID[33];
    memset(pVoiceID, 0, 33);
    int nRet =0;

    if (pVal_skillName) {
        env->ReleaseStringUTFChars(skillName, pVal_skillName);
    }
    if (pVal_skillID) {
        env->ReleaseStringUTFChars(skillID, pVal_skillID);
    }

    if (list_size > 0 && list_play_id) {
        for (int i = 0; i < list_size; ++i) {
            jstring playidstr = (jstring) env->GetObjectArrayElement(listPlayID, i);
            env->ReleaseStringUTFChars(playidstr, list_play_id[i]);
        }
        free(list_play_id);
    }

    jstring strVoiceID;
    ConvChar2JString(env, pVoiceID, strVoiceID);
    return strVoiceID;
}


//通用结果回调通知
void OnCommonRetCallback(const char *voice_id, int err_code) {

}

/**
 * 启用屏幕取词可见可答
 * return ret
 */
JNIEXPORT jint JNICALL
Java_com_tencent_xiaowei_sdk_XWSDKJNI_enableV2A(JNIEnv *env, jclass, jboolean enable) {
    __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "Java_com_tencent_xiaowei_sdk_XWSDKJNI_enableV2A");
    return xiaowei_enable_v2a(enable);
}

void on_xiaowei_uploadlist_callback(const char* voice_id, int errCode)
{
    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER, "on_xiaowei_setwordslist_callback NULL == tx_service");
        return;
    }

    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env) return;

    jclass cls = env->GetObjectClass(tx_service);
    jstring strVoiceID;
    ConvChar2JString(env, voice_id, strVoiceID);
    jmethodID methodID = env->GetMethodID(cls, "onUploadData", "(Ljava/lang/String;I)V");
    if (methodID) {
        env->CallVoidMethod(tx_service, methodID, strVoiceID, errCode);
    }

    env->DeleteLocalRef(cls);
    env->DeleteLocalRef(strVoiceID);
    if (needRelease)
        Util_ReleaseEnv();
}

/**
 * 设置词表
 */
JNIEXPORT jint JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_setWordslist(JNIEnv *env, jclass,
                                                                        jint type,
                                                                        jobjectArray arrayWordslist) {
    if (NULL == arrayWordslist) {
        __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "setWordslist NULL == arrayWordslist");
        return 0;
    }

    int list_size = env->GetArrayLength(arrayWordslist);
    if (list_size <= 0) {
        return 0;
    }

    char **words_list = (char **) malloc(list_size * sizeof(char *));
    if (words_list != NULL) {
        memset(words_list, 0, list_size * sizeof(char *));
        for (int i = 0; i < list_size; ++i) {
            jobject obj = env->GetObjectArrayElement(arrayWordslist, i);
            jstring words = (jstring) (obj);
            const char *szbuff = env->GetStringUTFChars(words, 0);
            int length = env->GetStringUTFLength(words);
            if (words && length > 0) {
                words_list[i] = (char *) malloc(sizeof(char) * (length + 1));
                memcpy(words_list[i], szbuff, length);
                words_list[i][length] = '\0';
            }
            env->ReleaseStringUTFChars(words, szbuff);
            env->DeleteLocalRef(obj);
        }
    }

    int ret = xiaowei_set_wordslist((XIAOWEI_WORDS_TYPE) type, words_list, list_size);

    if (words_list != NULL) {
        for (int i = 0; i < list_size; ++i) {
            if (words_list[i] != NULL)
                free(words_list[i]);
        }
        free(words_list);
    }
    return ret;
}

/**
 * 设置词表
 */
JNIEXPORT jstring JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_uploadDatalist(JNIEnv *env, jclass,
                                                                        jint type,
                                                                        jobjectArray arrayWordslist) {
    char szVoiceID[33] = {0};
    jstring strVoiceID;
    if (NULL == arrayWordslist) {
        __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "uploadDatalist NULL == arrayWordslist");
        ConvChar2JString(env, szVoiceID, strVoiceID);
        xiaowei_upload_data(szVoiceID, (XIAOWEI_UPLOAD_DATA_TYPE) type, nullptr, 0, on_xiaowei_uploadlist_callback);
        return strVoiceID;
    }

    int list_size = env->GetArrayLength(arrayWordslist);
    if (list_size <= 0) {
        __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "uploadDatalist data is null");
        ConvChar2JString(env, szVoiceID, strVoiceID);
        xiaowei_upload_data(szVoiceID, (XIAOWEI_UPLOAD_DATA_TYPE) type, nullptr, 0, on_xiaowei_uploadlist_callback);
        return strVoiceID;
    }

    char **words_list = (char **) malloc(list_size * sizeof(char *));
    if (words_list != NULL) {
        memset(words_list, 0, list_size * sizeof(char *));
        for (int i = 0; i < list_size; ++i) {
            jobject obj = env->GetObjectArrayElement(arrayWordslist, i);
            jstring words = (jstring) (obj);
            const char *szbuff = env->GetStringUTFChars(words, 0);
            int length = env->GetStringUTFLength(words);
            if (words && length > 0) {
                words_list[i] = (char *) malloc(sizeof(char) * (length + 1));
                memcpy(words_list[i], szbuff, length);
                words_list[i][length] = '\0';
            }
            env->ReleaseStringUTFChars(words, szbuff);
            env->DeleteLocalRef(obj);
        }
    }

    xiaowei_upload_data(szVoiceID, (XIAOWEI_UPLOAD_DATA_TYPE) type, words_list, list_size, on_xiaowei_uploadlist_callback);
    ConvChar2JString(env, szVoiceID, strVoiceID);
    if (words_list != NULL) {
        for (int i = 0; i < list_size; ++i) {
            if (words_list[i] != NULL)
                free(words_list[i]);
        }
        free(words_list);
    }
    return strVoiceID;
}

/**
 * 对上一次请求进行错误反馈
 * @param env
 * @return
 */
JNIEXPORT jint JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_errorFeedBack(JNIEnv *env, jclass, jstring voiceID) {
    
    const char *pVal_voiceID = NULL;
    if (voiceID) {
        pVal_voiceID = env->GetStringUTFChars(voiceID, 0);
    }
    
    int ret = xiaowei_error_feed_back(pVal_voiceID);

    if (pVal_voiceID) {
        env->ReleaseStringUTFChars(voiceID, pVal_voiceID);
    }

    return ret;
}

/**
 * 通用请求回调
 */
void on_request_cmd(const char* voice_id, int err_code, const char * json) {
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER,
                        "on_request_cmd voiceid:%s err_code:%d", voice_id,
                        err_code);

    if (NULL == tx_service) {
        __android_log_print(ANDROID_LOG_INFO, LOGFILTER,
                            "on_request_cmd NULL == tx_service");
        return;
    }

    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    if (!env) return;

    jclass cls = env->GetObjectClass(tx_service);
    jstring strVoiceID;
    ConvChar2JString(env, voice_id, strVoiceID);
    jstring strJson;
    ConvChar2JString(env, json, strJson);

    jmethodID methodID = env->GetMethodID(cls, "OnCommonRequest",
                                          "(Ljava/lang/String;ILjava/lang/String;)V");
    if (methodID) {
        env->CallVoidMethod(tx_service, methodID, strVoiceID, err_code, strJson);
    }

    env->DeleteLocalRef(cls);
    env->DeleteLocalRef(strVoiceID);
    env->DeleteLocalRef(strJson);

    if (needRelease)
        Util_ReleaseEnv();
}

JNIEXPORT jstring JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_requestCmd
        (JNIEnv *env, jclass, jstring cmd, jstring subCmd, jstring param) {
    char szVoiceID[33] = {0};

    const char *pVal_cmd = NULL;
    if (cmd) {
        pVal_cmd = env->GetStringUTFChars(cmd, 0);
    }
    const char *pVal_subCmd = NULL;
    if (subCmd) {
        pVal_subCmd = env->GetStringUTFChars(subCmd, 0);
    }
    const char *pVal_param = NULL;
    if (param) {
        pVal_param = env->GetStringUTFChars(param, 0);
    }

    xiaowei_request_cmd(szVoiceID, pVal_cmd, pVal_subCmd, pVal_param, on_request_cmd);

    jstring strVoiceID;
    ConvChar2JString(env, szVoiceID, strVoiceID);

    if (pVal_cmd) {
        env->ReleaseStringUTFChars(cmd, pVal_cmd);
    }
    if (pVal_subCmd) {
        env->ReleaseStringUTFChars(subCmd, pVal_subCmd);
    }
    if (pVal_param) {
        env->ReleaseStringUTFChars(param, pVal_param);
    }
    return strVoiceID;
}

/**
 * 上报播放状态
 * @param env
 * @param state
 * @return
 */
JNIEXPORT jint JNICALL
Java_com_tencent_xiaowei_sdk_XWSDKJNI_reportUserLog(JNIEnv *env, jclass, jobject log) {
    if (NULL == s_clsXWEventLogInfo ) {
        __android_log_print(ANDROID_LOG_ERROR, LOGFILTER,
                            "reportUserLog NULL == s_clsXWEventLogInfo ");
        return 0;
    }

    if (NULL == log) {
        __android_log_print(ANDROID_LOG_ERROR, LOGFILTER, "reportUserLog NULL == log");
        return 0;
    }

    jclass clsXWEventLogInfo = s_clsXWEventLogInfo;
    jfieldID id_voicId = env->GetFieldID(clsXWEventLogInfo, "voicId", "Ljava/lang/String;");
    jfieldID id_logData = env->GetFieldID(clsXWEventLogInfo, "logData", "Ljava/lang/String;");
    jfieldID id_event = env->GetFieldID(clsXWEventLogInfo, "event", "Ljava/lang/String;");
    jfieldID id_timeStamp = env->GetFieldID(clsXWEventLogInfo, "timeStamp", "J");

    jstring j_voicId = (jstring) env->GetObjectField(log, id_voicId);
    jstring j_logData = (jstring) env->GetObjectField(log, id_logData);
    jstring j_event = (jstring) env->GetObjectField(log, id_event);
    jlong j_timeStamp = (jlong) env->GetLongField(log, id_timeStamp);

    const char *pVal_voicId = NULL;
    if (j_voicId) {
        pVal_voicId = env->GetStringUTFChars(j_voicId, 0);
    }
    const char *pVal_logData = NULL;
    if (j_logData) {
        pVal_logData = env->GetStringUTFChars(j_logData, 0);
    }
    const char *pVal_event = NULL;
    if (j_event) {
        pVal_event = env->GetStringUTFChars(j_event, 0);
    }

    XIAOWEI_PARAM_LOG_REPORT logSt = {0};
    logSt.voice_id = pVal_voicId;
    logSt.log_data = pVal_logData;
    logSt.event = pVal_event;
    logSt.time_stamp_ms = j_timeStamp;


    int nRet = xiaowei_user_report(&logSt);

    if (pVal_voicId) {
        env->ReleaseStringUTFChars(j_voicId, pVal_voicId);
    }
    if (pVal_logData) {
        env->ReleaseStringUTFChars(j_logData, pVal_logData);
    }
    if (pVal_event) {
        env->ReleaseStringUTFChars(j_event, pVal_event);
    }

    return nRet;
}

/**
 * 设置设备可用状态
 * @param env
 * @return
 */
JNIEXPORT jint JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_setDeviceAvailability(JNIEnv *env, jclass, jint av)
{
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "setDeviceAvailability.");

    return xiaowei_set_device_availability(av);
}

JNIEXPORT jint JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_setDeviceScene(JNIEnv *env, jclass, jintArray sceneType, jobjectArray sceneInfo)
{
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "setDeviceScene.");

    int size = (NULL == sceneType) ? 0 : env->GetArrayLength(sceneType);
    int size2 = (NULL == sceneInfo) ? 0 : env->GetArrayLength(sceneInfo);
    if (size != size2) {
        __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "input size not match. type_size is %d. info size is %d", size, size2);
        return error_failed;
    }
     __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "setDeviceScene, size is %d", size);
    if (size > 0) {
        XIAOWEI_SCENE *scene = new XIAOWEI_SCENE[size];
        int* p = env->GetIntArrayElements(sceneType, 0);
        for (int i=0; i < size; ++i) {
            scene[i].type = *(p + i);
            jstring info = (jstring) env->GetObjectArrayElement(sceneInfo, i);
            if (info != NULL) {
                scene[i].info = (char *) env->GetStringUTFChars(info, 0);
            }
        }
        xiaowei_set_device_scene(scene,size);
        delete[] scene;
    } else {
        xiaowei_set_device_scene(nullptr, 0);
    }
    return error_null;
}

/**
 * 重置系统时钟
 * @return
 */
JNIEXPORT jint JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_manulSetClock(JNIEnv *env, jclass)
{
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "manulSetClock.");

    return xiaowei_manul_set_clock();
}

/**
 * 查询设备睡眠模式
 * @return
 */
JNIEXPORT jint JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_getSleepState(JNIEnv *env, jclass)
{
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "getSleepState.");

    return xiaowei_get_sleep_state();
}

/**
 * 临时屏蔽5分钟睡眠模式
 * @return
 */
JNIEXPORT jint JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_maskSleepModeTemporarily(JNIEnv *env, jclass)
{
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "maskSleepModeTemporarily.");

    return xiaowei_quit_sleep();
}

/**
 * 获取小微本地闹钟列表
 * @return clock_info json
 */
JNIEXPORT jstring JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_visitLocalClocks(JNIEnv *env, jclass)
{
  __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "visitLocalClocks.");

  std::string context;

  xiaowei_visit_local_clocks(&context, [](void *context, const char *json, int len) {
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER,
                        "on_visit_local_clocks json:%s len:%d", json,
                        len);

    if (NULL == tx_service || len == 0) {
      return ;
    }
    (*reinterpret_cast<std::string*>(context)) = std::string(json, len);

  });
  jstring str;
  ConvChar2JString(env, context.c_str() , str);
  return str;
}

/**
 * 获取小微云端ticket
 * @return errCode
 */
JNIEXPORT jint JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_getCloudTicket(JNIEnv *env, jclass, jobject listener)
{
  __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "getCloudTicket.");
  jobject global_listener = env->NewGlobalRef(listener);
  return xiaowei_get_cloud_ticket(global_listener, [](void *context, const char *ticket, int len, int expire_seconds) {
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER,
                        "onGetCloudTicket ticket:%s len:%d expire_seconds:%d", ticket,
                        len, expire_seconds);
    bool needRelease = false;
    JNIEnv *env = Util_CreateEnv(&needRelease);
    static auto onGetTicket_method_id = 
        env->GetMethodID(s_clsOnGetTicketListener, "onGetCloudTicket", "(Ljava/lang/String;I)V");
    auto listener = reinterpret_cast<jobject>(context);
    if (!env)  return;
    jstring str;
        ConvChar2JString(env, ticket , str);
    env->CallVoidMethod(listener, onGetTicket_method_id, str, expire_seconds);
    env->DeleteLocalRef(str);
    if (needRelease) Util_ReleaseEnv();
  });
}

/**
 * 创建解码工具
 * @return decoder handle
 */
JNIEXPORT jlong JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_decoderCreate(JNIEnv *env, jclass, jstring jkey)
{
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "decoderCreate.");
    if (!jkey)  return 0;
    const char* key = env->GetStringUTFChars(jkey,0);
    return reinterpret_cast<jlong>(xiaowei_decoder_create(key,strlen(key)));
}

/**
 * 解码
 * @return
 */
JNIEXPORT void JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_decoderDecode(JNIEnv *env, jclass, jlong jdecoder,
    jbyteArray jdata, jint jsize, jint joffset)
{
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "decodeDecode.");
    jbyte* jniBuf = (env)->GetByteArrayElements(jdata, JNI_FALSE);
    xiaowei_decoder_decode(reinterpret_cast<void*>(jdecoder), reinterpret_cast<char*>(jniBuf), jsize, joffset);
    env->ReleaseByteArrayElements(jdata, jniBuf, 0);
}

/**
 * 销毁解码工具
 * @return decoder handle
 */
JNIEXPORT void JNICALL Java_com_tencent_xiaowei_sdk_XWSDKJNI_decoderDestroy(JNIEnv *env, jclass,jlong jdecoder)
{
    __android_log_print(ANDROID_LOG_DEBUG, LOGFILTER, "decoderDestroy.");
    xiaowei_decoder_destroy(reinterpret_cast<void*>(jdecoder));
}
#ifdef __cplusplus
}
#endif
