/*
 * Tencent is pleased to support the open source community by making  XiaoweiSDK Demo Codes available.
 *
 * Copyright (C) 2017 THL A29 Limited, a Tencent company. All rights reserved.
 *
 * Licensed under the MIT License (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://opensource.org/licenses/MIT
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 *
 */
package com.tencent.xiaowei.def;

/**
 * 通道层常量定义
 */
public class XWCommonDef {

    /**
     * 语音请求的事件定义
     */
    public interface XWEvent {
        /**
         * 空闲
         */
        int ON_IDLE = 0;
        /**
         * 请求开始
         */
        int ON_REQUEST_START = 1;
        /**
         * 检测到开始说话
         */
        int ON_SPEAK = 2;
        /**
         * 检测到静音(only@device has not txca_device_local_vad)
         */
        int ON_SILENT = 3;
        /**
         * 识别文本实时返回
         */
        int ON_RECOGNIZE = 4;
        /**
         * 请求收到响应
         */
        int ON_RESPONSE = 5;
        /**
         * 收到TTS推送
         */
        @Deprecated
        int ON_TTS_PUSH = 6;
        /**
         * 请求收到中间响应
         */
        int ON_RESPONSE_INTERMEDIATE = 7;
    }

    public interface DEVICE_INFO_PROP {
        long USE_LOCAL_VAD = 0x0000000000000001;
        long SUPPROT_GPS = 0x0000000000000002;
        long NO_TTS = 0x0000000000000004;
        long USE_LOCAL_TTS = 0x0000000000000008;   
        long ONLY_VAD  = 0x0000000000000010;   
    }

    /**
     * 播放资源列表类型
     */
    public interface ResourceListType {

        /**
         * 默认的列表，当前列表。
         */
        int DEFAULT = 0;

        /**
         * 这个场景的历史列表
         */
        int HISTORY = 1;
    }

    /**
     * 播放资源入队行为定义
     */
    public interface PlayBehavior {
        /**
         * 中断当前播放，替换列表
         */
        int REPLACE_ALL = 0;
        /**
         * 拼接到列表队头
         */
        int ENQUEUE_FRONT = 1;
        /**
         * 拼接到列表队尾
         */
        int ENQUEUE_BACK = 2;
        /**
         * 不中断当前播放的资源，替换列表
         */
        int ENQUEUE_REPLACE = 3;
        /**
         * 不中断播放，更新列表中某些播放资源的url和quality字段信息
         */
        int ENQUEUE_UPDATE = 4;
        /** @deprecated
         * 不中断播放，更新列表中某些播放资源的url和所有description
         */
        int ENQUEUE_UPDATE_ALL = 5;
        /**
         * 从当前列表中移除这些元素
         */
        int REMOVE = 6;

    }

    public interface ReponseTypeDef {
        int TYPE_UNKNOW = 0;
        int TYPE_CARD = 1;                 /// card
        int TYPE_WEATHER = 2;              /// weather: json
        int TYPE_GAME = 3;                 /// game
        int TYPE_CLOCK = 4;                /// reminder: json
        int TYPE_ENGLISH = 5;              /// english reading/media: json
        int TYPE_LOCAL_SKILL = 6;          /// local skill: intent and slot info
        int TYPE_MSGBOX = 7;               /// message box: json
        int TYPE_FETCH_DEVICE_INFO = 8;    /// fetch device info: json
        int TYPE_NEWS = 9;                 /// news: json
        int TYPE_WIKI = 10;                /// wiki: json
        int TYPE_MINI_APP = 11;            /// miniapp: json
        int TYPE_VOIP = 12;                /// voip
        int TYPE_IOT_DEVICE_LIST = 13;     /// IOT device list
        int TYPE_IOT_ORG_REQUEST = 14;     /// IOT request text
        int TYPE_TANSLATE = 15;            /// translate: json
        int TYPE_STOCK = 16;               /// stock json
        int TYPE_DEGNOISEd = 17;           /// blue-tooth degnoise: json
        int TYPE_MINI_PROGRAM = 18;        /// voice control of mini program
        int TYPE_PHOTO = 19;               /// photo: json
        int TYPE_XM_LOGIN = 20;            /// ximalaya login: json
        int TYPE_XM_QR_CODE = 21;          /// ximalaya query qr status: json
        int TYPE_XM_LOGOUT = 22;           /// ximalaya logout: json
        int TYPE_LOCAL_APP = 23;           /// open local app: json
    }

    public interface SkillIdDef {
        /**
         * 通用控制
         */
        String SKILL_ID_GLOBAL = "8dab4796-fa37-4114-0000-7637fa2b0000";
        /**
         * 天气
         */
        String SKILL_ID_WEATHER = "8dab4796-fa37-4114-0012-7637fa2b0003";

        /**
         * 音乐
         */
        String SKILL_ID_MUSIC = "8dab4796-fa37-4114-0011-7637fa2b0001";

        /**
         * FM
         */
        String SKILL_ID_FM = "8dab4796-fa37-4114-0024-7637fa2b0001";

        /**
         * 新闻
         */
        String SKILL_ID_New = "8dab4796-fa37-4114-0019-7637fa2b0001";

        /**
         * 搜狗百科-历史上的今天
         */
        String SKILL_ID_WIKI_HISTORY = "8dab4796-fa37-4114-0027-7637fa2b0001";

        /**
         * AI-LAB百科
         */
        String SKILL_ID_WIKI_AI_LAB = "8dab4796-fa37-4114-0020-7637fa2b0001";

        /**
         * 搜狗百科-当前时间
         */
        String SKILL_ID_WIKI_Time = "8dab4796-fa37-4114-0028-7637fa2b0001";

        /**
         * 搜狗-计算器
         */
        String SKILL_ID_WIKI_Calculator = "8dab4796-fa37-4114-0018-7637fa2b0001";

        /**
         * 微信闲聊
         */
        String SKILL_ID_WX_Chat = "8dab4796-fa37-4114-0029-7637fa2b0001";

        /**
         * 闹钟
         */
        String SKILL_ID_ALARM = "8dab4796-fa37-4114-0012-7637fa2b0001";

        /**
         * 闹钟触发场景
         */
        String SKILL_ID_TRIGGER_ALARM = "8dab4796-fa37-4114-0036-7637fa2b0001";

        /**
         * 翻译
         */
        String SKILL_ID_TRANSLATE = "8dab4796-fa37-4114-0030-7637fa2b0001";

        /**
         * QQ电话
         */
        String SKILL_ID_QQ_CALL = "8dab4796-fa37-4114-0001-7637fa2b0001";

        /**
         * QQ消息
         */
        String SKILL_ID_QQ_MSG = "8dab4796-fa37-4114-0002-7637fa2b0001";

        /**
         * 导航
         */
        String SKILL_ID_NAV = "8dab4796-fa37-4114-0015-7637fa2b0001";

        /**
         * 视频
         */
        String SKILL_ID_VIDEO = "8dab4796-fa37-4114-0026-7637fa2b0001";

        /**
         * 微信消息
         */
        String SKILL_ID_WECHAT_MSG = "8dab4796-fa37-4114-0002-7637fa2b0002";

        /**
         * 微信小程序
         */
        String SKILL_ID_MINI_APP = "8dab4796-fa37-4114-0042-7637fa2b0001";

        /**
         * 视频通话
         */
        String SKILL_ID_WX_VOIP = "8dab4796-fa37-4114-0001-7637fa2b0002";

        //FM类
        /**
         * 热门
         */
        String SKILL_ID_FM_HOT_PRO = "8dab4796-fa37-4114-0024-7637fa2b0002";
        /**
         * 传统电台
         */
        String SKILL_ID_FM_TRA_PRO = "8dab4796-fa37-4114-0024-7637fa2b0003";
        /**
         * 电子书
         */
        String SKILL_ID_FM_NOVEL = "8dab4796-fa37-4114-0024-7637fa2b0004";
        /**
         * 故事
         */
        String SKILL_ID_FM_STORY = "8dab4796-fa37-4114-0024-7637fa2b0005";
        /**
         * 笑话
         */
        String SKILL_ID_FM_JOKE = "8dab4796-fa37-4114-0024-7637fa2b0006";
        /**
         * 新闻
         */
        String SKILL_ID_FM_NEWS = "8dab4796-fa37-4114-0024-7637fa2b0007";
        /**
         * 评书
         */
        String SKILL_ID_FM_PINGSHU = "8dab4796-fa37-4114-0024-7637fa2b0008";
        /**
         * 相声
         */
        String SKILL_ID_FM_XIANGSHENG = "8dab4796-fa37-4114-0024-7637fa2b0009";

        /**
         * IOT主控
         */
        String SKILL_ID_IOT_MASTER_DEVICE = "8dab4796-fa37-4114-ffff-000000000000";

        /**
         * 设备被控
         */
        String SKILL_ID_IOT_CONTROLLER_DEVICE = "8dab4796-fa37-4114-0037-7637fa2b0001";
        /**
         * 股票
         */
        String SKILL_ID_STOCK = "8dab4796-fa37-4114-0033-7637fa2b0001";

        /**
         * 对不起，我没有明白你的意思
         */
        String SKILL_ID_Unknown = "8dab4796-fa37-4114-ffff-ffffffffffff";


        /**
         * 打开应用
         */
        String SKILL_OPEN_APP = "8dab4796-fa37-4114-0042-7637fa2b0003";

        /**
         * 分享
         */
        String SKILL_ID_SHARE = "8dab4796-fa37-4114-1000-7637fa2b0001";
    }

    /**
     * 请求类型
     */
    public interface RequestType {
        /**
         * 语音请求
         */
        int VOICE = 0;
        /**
         * 文本请求
         */
        int TEXT = 1;
        /**
         * 请求TTS资源
         */
        int ONLY_TTS = 2;
        /**
         * 请求意图，未实现
         */
        int Intent = 3;
        /**
         * 唤醒校验
         */
        int WAKEUP_CHECK = 4;
    }

    /**
     * 唤醒类型
     */
    public interface WAKEUP_FLAG {
        /**
         * 这个请求不是云端唤醒请求，无需处理
         */
        public final static int WAKEUP_FLAG_NO = 0;
        /**
         * 唤醒失败，唤醒校验结束，就当什么都没发生
         */
        public final static int WAKEUP_FLAG_FAIL = 1;

        /**
         * 唤醒成功，唤醒校验结束
         */
        public final static int WAKEUP_FLAG_SUC = 2;
        
        /**
         * 唤醒成功，并且直接理解了用户的需求
         */
        public final static int WAKEUP_FLAG_SUC_RSP = 3;

        /**
         * 唤醒成功，并且用户还在继续说话，不知道用户说完没有
         */
        public final static int WAKEUP_FLAG_SUC_CONTINUE = 4;
    }


    /**
     * 唤醒类型
     */
    public interface WAKEUP_TYPE {
        /**
         * 默认，纯本地唤醒或者按键唤醒
         */
        public final static int WAKEUP_TYPE_DEFAULT = 0;

        /**
         * 纯本地唤醒但是需要云端过滤掉唤醒词
         */
        public final static int WAKEUP_TYPE_LOCAL_WITH_TEXT = 2;

        /**
         * 全双工模式
         */
        public final static int WAKEUP_TYPE_LOCAL_WITH_FREE = 3;
    }

    /**
     * 远近场类型
     */
    public interface PROFILE_TYPE {
        /**
         * 远场
         */
        public final static int PROFILE_TYPE_FAR = 0;
        /**
         * 近场
         */
        public final static int PROFILE_TYPE_NEAR = 1;

    }

    //控制指令
    public interface CMD {
    //声音相关
    public final static int
        VOL_ADD         = 1000000, //音量增加，具体值为controlValue，单位为百分数，-100~+100
        VOL_SET_TO      = 1000001, //设置音量到某个位置
        MUTE            = 1000002, //音箱静音
        UNMUTE          = 1000003, //取消音箱静音
        CLOSE_MIC       = 1000004, //麦克风关闭
        OPEN_MIC        = 1000005, //麦克风打开

        //播放控制相关
        PLAY            = 1000010, //播放/恢复播放
        STOP            = 1000011, //停止(停止播放之后就无法再恢复播放了，再次播放就从头开始播)
        PAUSE           = 1000012, //暂停
        REPLAY          = 1000013, //从头播放该资源
        PLAY_PREV       = 1000014, //上一首
        PLAY_NEXT       = 1000015, //下一首
        PLAY_RES_ID     = 1000016, //播放某个id的资源，id在controlValue里
        OFFSET_ADD      = 1000017, //快进/快退一段时间，值在controlValue里，单位毫秒，正值快进，负值快退
        OFFSET_SET      = 1000018, //跳转到某一播放时刻
        SET_PLAY_MODE   = 1000019, //设置播放模式，值在controlValue里，0:顺序播放，1:列表循环，2:随机播放(播完列表就结束）3:随机循环播放 4:单曲循环
        SET_PLAY_SPEED   = 1000030,//设置播放速度

        //设备相关
        BRIGHT_ADD      = 1000050, //亮度增加 value: -100~100
        BRIGHT_SET      = 1000051, //亮度设置 value: 0~100
        REPORT_STATE    = 1000100, //后台需要设备尽快上报当前的播放状态
        SHUTDOWN        = 1000101, //关机
        REBOOT          = 1000102, //重启
        BLE_MODE_ON     = 1000103, //进入蓝牙模式
        BLE_MODE_OFF    = 1000104, //退出蓝牙模式
        DUPLEX_MODE_ON  = 1000105, //进入全双工模式
        DUPLEX_MODE_OFF = 1000106, //退出全双工模式

        //favorite
        FAVORITE        = 1100010, //收藏某首歌，control value为资源playid
        UN_FAVORITE     = 1100011, //取消收藏

        // 技能闲逛
        CMD_V2A         = 1200001;   /// V2A matched, controlValue is json {"match_word":"xxx"}

    }

    /**
     * 请求可带的参数，可以复合使用
     */
    public interface REQUEST_PARAM {
        /**
         * 不使用小微的Vad，使用本地的Vad，带上这个标记后，需要在说话结束后主动通知SDK voiceRequestEnd=true。
         */
        public static final int REQUEST_PARAM_USE_LOCAL_VAD = 0x1;

        /**
         * 使用GPS的位置，如果设为true，小微将在发送请求时回调GPS接口来获取GPS信息，该接口将同步等待，所以返回一定要快
         */
        public static final int REQUEST_PARAM_GPS = 0x2;

        /**
         * 无需云端下发TTS，使用纯本地TTS
         */
        public static final int REQUEST_PARAM_NO_TTS = 0x4;

        /**
         * 使用小微官方的本地TTS包
         */
        public static final int REQUEST_PARAM_USE_LOCAL_TTS = 0x8;

        /**
         * 这次请求不识别，只进行小微的Vad
         */
        public static final int REQUEST_PARAM_ONLY_VAD = 0x10;

    }

    /**
     * 资源格式定义
     */
    public interface ResourceFormat {
        /**
         * URL类型，
         */
        int URL = 0;
        /**
         * 文本类型
         */
        int TEXT = 1;
        /**
         * TTS类型，ID=resID
         */
        int TTS_ID = 2;
        /**
         * 本地文件类型
         */
        int TTS_URL = 3;
        /**
         * 位置类型
         */
        int LOCATION = 4;
        /**
         * 指令类型，ID=指令ID；content=指令内容
         */
        int COMMAND = 5;
        /**
         * 原始语义意图信息，content=语义json
         */
        int INTENT = 6;
        /**
         * 提醒类
         */
        int REMIND = 7;

        /**
         * 分享图片
         */
        int IMAGE_SHARE = 10;
        /**
         * 分享文件
         */
        int FILE_SHARE = 11;
        /**
         * 分享位置
         */
        int POI_SHARE = 12;
        /**
         * 分享视频
         */
        int VIDEO_SHARE = 13;
        /**
         * 分享链接
         */
        int URL_SHARE = 14;

        /**
         * 本地资源
         */
        int LOCAL = 89;
        /**
         * 未知类型
         */
        int UNKNOW = 99;
    }

    /**
     * 上报状态类型定义
     */
    public interface PlayState {
        /**
         * 预加载
         */
        int PRELOAD = 0;
        /**
         * 开始播放
         */
        int START = 1;
        /**
         * 暂停播放
         */
        int PAUSED = 2;
        /**
         * 播放停止，正常播放完毕使用这个，但是还在播放skill下
         */
        int STOPED = 3;
        /**
         * 播放中断，播放出错或者用户主动中断，
         */
        int FINISHED = 4;
        /**
         * 空闲，没有做任何事
         */
        int IDLE = 5;
        /**
         * 继续播放
         */
        int RESUME = 6;
        /**
         * 放弃
         */
        int ABORT = 11;
        /**
         * 播放出错
         */
        int ERROR = 12;
    }

    /**
     * 播放模式定义
     */
    public interface PlayMode {
        /**
         * 顺序播放
         */
        int SEQUENTIAL_NO_LOOP = 0;
        /**
         * 列表循环
         */
        int SEQUENTIAL_LOOP_PLAYLIST = 1;
        /**
         * 随机，不循环
         */
        int SHUFFLE_NO_LOOP = 2;
        /**
         * 随机列表循环
         */
        int SHUFFLE_LOOP_PLAYLIST = 3;
        /**
         * 单曲循环
         */
        int LOOP_SINGLE = 4;
    }

    /**
     * 设备可用状态定义
     */
    public interface DeviceAvailability {
        /**
         * 当前设备空闲，小微可以做任何事
         */
        int DEFAULT = 0x0;
        /**
         * 麦克风不可用
         */
        int MIC_NOT_AVAILABLE = 0x1;
        /**
         * 音频输出不可用
         */
        int SPEAKER_NOT_AVAILABLE = 0x2;
        /**
         * 勿扰模式
         */
        int INITIATIVE_SPEAK = 0x4;
        /**
         * 禁止远程控制
         */
        int DISABLE_REMOTE_CONTROL = 0x8;
        /**
         * 设备进入小程序模式
         */
        int MINI_PROGRAM_MODE = 0x10;
    }

    /**
     * 设备播放的资源类型，上报使用
     */
    public interface ReportPlayResourceType {
        /**
         * 正在播放URL类型的媒体资源（TTS除外）
         */
        int URL = 0;
        /**
         * 正在显示文本资源
         */
        int TEXT = 1;
        /**
         * 正在播放TTS资源（可能来自本地，也可能是云端推送的pcm包，总之抽象成本地资源）
         */
        int TTS = 2;
        /**
         * 正在播放URL类型的TTS
         */
        int TTS_URL = 3;
    }

    /**
     * QQ音乐品质定义
     */
    public interface PlayQuality {
        /**
         * 流畅
         */
        int PLAY_QUALITY_LOW = 0;

        /**
         * 标准
         */
        int PLAY_QUALITY_NORMAL = 1;

        /**
         * 高品质
         */
        int PLAY_QUALITY_HIGH = 2;

        /**
         * 无损
         */
        int PLAY_QUALITY_LOSSLESS = 3;
    }

    /**
     * 闹钟/提醒操作类型定义
     */
    public interface AlarmOptType {
        /**
         * 新增闹钟
         */
        int ALARM_OPT_TYPE_ADD = 1;
        /**
         * 更新闹钟
         */
        int ALARM_OPT_TYPE_UPDATE = 2;
        /**
         * 删除闹钟
         */
        int ALARM_OPT_TYPE_DELETE = 3;
    }

    /**
     * 可见可达词表类型
     */
    public interface WordListType {
        /**
         * 可见可答屏幕词表，常用于屏幕上的按钮，设置后是一次性的
         * 对应的words数据直接是词表，例如words[0] = "确定"，words[1] = "取消"
         */
        int WORD_TYPE_COMMAND = 0;
        /**
         * 本地相册的文件名，用于相册技能
         * 对应的words数据直接是相册名称，例如words[0] = "我的相册"，words[1] = "澳洲旅行"
         */
        int PHOOTO_FILE = 4;
    }

    /**
     * 本地数据上传类型
     */
    public interface DataListType {
        /**
         * 本地APP列表，数据格式为json，每个数据代表一个APP
         * {"appName":"","appNickName":["name1","name2"],"appId":"app_id"}
         * 其中"appName"和"appId"是必填项。
         */
        int LOCAL_APP = 0;

        /**
         * 本地联系人
         */
        int LOCAL_CONTACT = 1;
    }

    /**
     * 全局通用错误码
     */
    public interface ErrorCode {
        /** global error */
        /**
         * 成功
         */
        int ERROR_NULL_SUCC = 0;
        /**
         * 失败
         */
        int ERROR_FAILED = 1;
        /**
         * 重复初始化SDK
         */
        int ERROR_SDK_REINIT = 5002;
        /**
         * 初始化路径不可达
         */
        int ERROR_INIT_PATH_INVALID = 5003;
        /**
         * 初始化账号信息格式错误
         */
        int ERROR_ACCOUNT_INVALID = 5004;
        /**
         * 参数非法
         */
        int ERROR_INVALID_PARAM = 5005;
        /**
         * SKD还没登录成功
         */
        int ERROR_SDK_UNINIT = 5006;
        /**
         * 分配内存失败
         */
        int ERROR_MEM_ALLOC = 5007;
        /**
         * 还没登录成功
         */
        int ERROR_LOGIN_NOT_SUCCESS = 5008;
        /**
         * 接口暂未实现
         */
        int ERROR_NOT_IMPL = 5009;
        /**
         * 逻辑错误
         */
        int ERROR_LOGIC = 5010;
        /**
         * 无权限
         */
        int ERROR_PERMISSION = 5011;


        /** login error */
        /**
         * 签名错误
         */
        int ERROR_INVALID_SIGN = -5;
        /**
         * 设备未预注册
         */
        int ERROR_NOT_REGISTER = -17;



        /** network and auth error */
        /**
         * 网络底层错误
         */
        int ERROR_NET_SYS = 31;
        /**
         * 网络超时
         */
        int ERROR_NET_TIMEOUT = 33;
        /**
         * 长时间未使用，下次恢复正常
         */
        int ERROR_SESSION_TIMEOUT = 35;
    }

    /**
     * 错误码定义
     */
    public interface XWeiErrorCode {
        /**
         * 成功
         */
        int SUCCESS = 0;

        /**
         * 失败
         */
        int ERR_FAILED = 1;

        /**
         * 请求解包错误
         */
        int PARSE_REQ_ERR = 10000;

        /**
         * 空的语音数据
         */
        int EMPTY_VOICE_DATA = 10001;

        /**
         * 语音转文本失败
         */
        int VOICE_TO_TEXT_ERR = 10002;

        /**
         * 语义分析失败
         */
        int TEXT_ANALY_ERR = 10003;

        /**
         * 文本转语音失败
         */
        int TEXT_TO_VOICE_ERR = 10004;

        /**
         * 无法获取资源
         */
        int RESOURCE_INVALID = 10005;

        /**
         * 被限制接入的设备
         */
        int INVALID_DEVICE = 10006;

        /**
         * 语音输入过长
         */
        int VOICE_TOO_LONG = 10007;

        /**
         * 唤醒后一直没说话，触发超时
         */
        int VOICE_TIMEOUT = 10008;

        /**
         * 多轮对话过程中场景切换了
         */
        int SCENE_CHANGE = 10009;

        /**
         * 没有命中任何Skill
         */
        int NOT_MATCH_SKILL = 10010;

        /**
         * 说完话后等待小微响应最终结果超时
         */
        int WAIT_RESULT_TIMEOUT = 10017;

        /**
         * 尾部静音检测超时
         */
        int AUDIO_END_TIMEOUT = 10018;

        /**
         * NLP工作正常，但是无法理解
         */
        int DONOT_UNDERSTAND = 20001;

        /**
         * 小微明白你的意思，但是当前设备不支持此操作
         */
        int NOT_SUPPORT = 20002;

        /**
         * 后台处理流程错误
         */
        int INTERNAL_ERROR = 20003;

        /**
         * 语音请求参数非法
         */
        int INVALID_PARAM = 20004;

        /**
         * 登录态错误，未授权
         */
        int AUTH_ERROR = 20005;

        /**
         * 逻辑错误，如设置一个昨天的闹钟
         */
        int LOGIC_ERROR = 20006;

        /**
         * 请求不和谐，命中黄反等敏感问题
         */
        int REQ_ILLEGAL = 20007;

        /**
         * 请求太频繁，被后台加入黑名单
         */
        int REQ_TOO_RREQUENTLY  = 21031;

        int MUSIC_PARAMETER          = -100000; // 参数错误
        int NOT_LOGIN                = -100032; // 未登录点歌
        int NOT_VIP                  = -100033; // 付费歌曲，非会员不能播放
        int NOT_FIND_SINGER_SONG     = -100034; // 以歌手-歌曲方式点歌，未找到结果
        int SINGER_SONG_COPYRIGHT    = -100035; // 因歌曲没有版权不能播放
        int NOTHING_FIND             = -100036; // 未查询到歌曲
        int NOT_HOMEUSER             = -100037; // 因海外地区不能播放
        int NEED_BUY                 = -100038; // 因歌曲为数字专辑歌曲不能播放
        int UNPLAYABLE_OTHER         = -100039; // 其他原因不能播放
        int CHANGE_SINGER            = -100040; // 换歌手失败
        int ACCOUNT_VERIFY           = -100041; // 账户信息验证错误
        int SYS_PARAMETER            = -200001; // 参数错误
        int SYS_PRE_GET              = -200010; // 预拉取请求音乐失败
        int SYS_PICK_PARA            = -200012; // 点播参数错误
        int SYS_PICK_ID_ERR          = -200013; // 点播ID不正确
        int SYS_PICK_FAILD           = -200015; // 点播请求音乐失败
        int SYS_PLAY_LIST_FIAL       = -200017; // 查询列表请求音乐失败
        int SYS_ITEM_DETAIL_PARA     = -200021; // 查详情参数错误
        int SYS_ITEM_ID_ERR          = -200022; // 查详情ID不正确
        int SYS_PLAY_LIST_NO_MORE    = -200030; // 查询列表没更多了
        int SYS_PRE_GET_PARAMETER    = -200036; // 预拉取参数错误
        int SYS_PLAY_LIST_ID         = -200044; // 查询列表参照ID不存在
        int SYS_PRE_GET_NO_MORE      = -200049; // 预拉取没更多了
        int SYS_DELETITEM_PARA       = -200051; // 列表中删除元素参数错误
        int SYS_DELET_ITEM_FAILD     = -200052; // 列表中删除元素操作失败
        int SYS_NO_HISTORY_LIST      = -200053; // 不存在历史列表
        int SYS_CLEAR_PLAY_LIST      = -200054; // 解绑清除数据操作失败
        int SYS_GET_HISTORY_LIST     = -200055; // 查询历史列表请求音乐失败
        int SYS_FAVORITE_PARAM       = -200060; // 操作收藏没带RESID
    }

}
