/*
 * Tencent is pleased to support the open source community by making  XiaoweiSDK Demo Codes available.
 *
 * Copyright (C) 2017 THL A29 Limited, a Tencent company. All rights reserved.
 *
 * Licensed under the MIT License (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://opensource.org/licenses/MIT
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 *
 */
package com.tencent.xiaowei.sdk;

import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import com.tencent.xiaowei.info.DeviceBinderInfo;
import com.tencent.xiaowei.info.DeviceLocationInfo;
import com.tencent.xiaowei.info.DeviceOTAInfo;
import com.tencent.xiaowei.info.DeviceProfile;
import com.tencent.xiaowei.info.XWEventLogInfo;
import com.tencent.xiaowei.info.XWLoginStatusInfo;
import com.tencent.xiaowei.info.XWPlayStateInfo;
import com.tencent.xiaowei.info.XWRequestInfo;
import com.tencent.xiaowei.info.XWResponseInfo;
import com.tencent.xiaowei.info.XWTTSDataInfo;
import com.tencent.xiaowei.util.Singleton;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * SDK的jni层，这个文件用户自己一般不应修改和关注。
 */
public class XWSDKJNI {
    private static final String TAG = "DEVICE_SDK_JNI";

    private static final Singleton<XWSDKJNI> sSingleton = new Singleton<XWSDKJNI>() {
        @Override
        protected XWSDKJNI createInstance() {
            return new XWSDKJNI();
        }
    };

    public static XWSDKJNI getInstance() {
        return sSingleton.getInstance();
    }

    static {
        System.loadLibrary("c++_shared");
        System.loadLibrary("owl");
        System.loadLibrary("sqlite");
        System.loadLibrary("ilink_xlog");
        System.loadLibrary("ilink_network");
        System.loadLibrary("ilink_dev");
        System.loadLibrary("private_protobuf");
        System.loadLibrary("ilink_im");
        System.loadLibrary("confService");
        System.loadLibrary("xiaoweiSDK");
        System.loadLibrary("ilinkJni");
    }

    private long mSelfDin;
    private Handler mHandler = new Handler(Looper.getMainLooper());
    // 联系人列表
    protected static final ArrayList<DeviceBinderInfo> mBinderList = new ArrayList<>();

    private HashMap<String, String[]> mLogMap = new HashMap<>(1);

    static void postMain(Runnable runnable) {
        getInstance().mHandler.post(runnable);
    }

    static void postMainDelay(Runnable runnable, long delayMillis) {
        getInstance().mHandler.postDelayed(runnable, delayMillis);
    }

    static void removeMainRunnable(Runnable runnable) {
        getInstance().mHandler.removeCallbacks(runnable);
    }

    /** voip begin*/
    public static int field_remoteImgLength = 0;
    public static int field_remoteImgHeight = 0;
    public static int field_remoteImgWidth  = 0;
    public static int field_remoteImgMember  = -1;  //member id
    public static int field_remoteOutFormat = -1;
    public static int field_remoteImgOrien = -1;

    //video
    public static int field_HWEncW = 0;
    public static int field_HWEncH = 0;

    public static class HWEncInfo{
        public int w = 0;
        public int h = 0;

        public HWEncInfo(int w,int h){
            this.h = h;
            this.w = w;
        }
    }

    public static HWEncInfo getHWEncInfo(){
        return new HWEncInfo(
            XWSDKJNI.field_HWEncW,
            XWSDKJNI.field_HWEncH);
    }

    public static class VideoInfo{
        public int h = 0;
        public int w = 0;
        public int len = 0;
        public int memberId = -1;
        public int format = -1;
        public int orien = -1;

        public VideoInfo(int h,int w,int l,int memberId, int format,int orien){
            this.h = h;
            this.w = w;
            this.len = l;
            this.memberId = memberId;
            this.format = format;
            this.orien = orien;
        }
    }

    public static VideoInfo getVideoInfo(){
        return new VideoInfo(
            XWSDKJNI.field_remoteImgHeight,
            XWSDKJNI.field_remoteImgWidth,
            XWSDKJNI.field_remoteImgLength,
            XWSDKJNI.field_remoteImgMember,
            XWSDKJNI.field_remoteOutFormat,
            XWSDKJNI.field_remoteImgOrien);
    }

    public static int GetDecodeVideoData(byte [] iImg)
    {

        iImg[3] = 0x00;
        iImg[2] = iImg[1] = 0x00;
        iImg[0] = 0x02;

        //奇偶调用，不知道什么东西
        byte mark = 2;
        while (mark-- >= 0) {
            iImg[0] = mark;
            int ret = GetVideoData(iImg);
            int len = field_remoteImgLength;
            int height = field_remoteImgHeight;
            int width = field_remoteImgWidth;
            int mid = field_remoteImgMember;
            int format = field_remoteOutFormat;
            int orien = field_remoteImgOrien;

            // steveshi want to do not check this
            if (ret <= 0 || len <= 0) {
//            if (ret <= 0 || len <= 0 || height <= 0 || width <= 0) {
                continue;
            }
            return ret;
        }
        return 0;
    }

    private void onVoipInvited(String fromUsername, int type)
    {
        DeviceSDK.getInstance().onVoipInvite(fromUsername, type);
    }

    private void onVoipJoinRoom(int result, boolean isMaster)
    {
        DeviceSDK.getInstance().onJoinRoom(result, isMaster);
    }

    private void onVoipInviteResult(int result)
    {
        DeviceSDK.getInstance().onInviteResult(result);
    }

    private void onVoipCancel(boolean isMaster)
    {
        DeviceSDK.getInstance().onVoipCancel(isMaster);
    }

    private void onVoipTalking()
    {
        DeviceSDK.getInstance().onVoipTalking();
    }

    private void onVoipFinish()
    {
        DeviceSDK.getInstance().onVoipFinish();
    }

    private void onVoipParamChange(int sampleRate, int sampleLenInms, String extendPB)
    {
        DeviceSDK.getInstance().onVoipParamChange(sampleRate, sampleLenInms, extendPB);
    }

    private void onVoipMemberChange(int[] members, String[] usernames)
    {
        DeviceSDK.getInstance().onVoipMemberChange(members, usernames);
    }

    private void onVoipAVMemberChange(int[] audioMembers, int[] videoMembers)
    {
        DeviceSDK.getInstance().onVoipAVMemberChange(audioMembers, videoMembers);
    }

    public static native int startVoipService(int audioType, int closeAV, int videoRatio, int defaultVideoLength, int cpuCoreNum, int cpuFreq, int cpuArch);
    public static native int createVoipCall(String ilinkId, String appId, int type);
    public static native int hangUpVoipCall();
    public static native int stopVoipService();
    public static native int answerVoipCall();

    public static native int SendAudioData(byte[] buffer, int len, int delay);

    //send camera video frame to encode&send to network
    public static native int SendVideoData(byte[] img, int len, int w, int h, int f);

    //get received audio data to playout:
    public static native int GetAudioData(byte[] playAudio, int len);

    public static native int voipVideoPreEncProcessCloud(byte[] recordData,int len,int videoWidth,int videoHeight,int format,byte[] encImg);

    //get decoded video frame to render:
    public static native int GetVideoData(byte[] iImg);

    public static native int switchVoipAV(boolean isOpenAudio, boolean isOpenVideo);

    public static native int setVoipAppCmd(int type, byte[] buffer, int len);

    public static native int getVoipVoiceActivity(int memberId);

    public static native int[] getVoipVadMember();

    public static native int voipSetVideoResolution(byte[] pbBuf, int len);
    /** voip end*/

    /** voice link */
    /**
     * wifi配网信息解析，启动解析模块
     *
     * @param sampleRate 声波配网, 设备实际录音的采样率，填的不对，会导致解声波信息失败。
     * @return
     */
    static native int startWifiDecoder(int sampleRate);

    /**
     * 填充wav数据。
     *
     * @param wav wav 是PCM 16bit 单声道的，size < 2048Byte
     */
    static native void fillVoiceWavData(byte[] wav);

    /**
     * wifi配网信息解析，停止解析模块
     *
     * @return
     */
    static native int stopWifiDecoder();
    /** voice link end */

    ///////////////////////////////////////////////////native//////////////////////////////////////////
    /**
     * 先初始化JNI，再干别的事
     */
    public native void initJNI(int log_level);

    /**
     * 初始化sdk
     *
     * @param Platform       设备平台名称，应该填写"Android"
     * @param license        设备授权码
     * @param serialNumber   设备序列号，
     * @param productID      产品ID，厂商向腾讯开放平台部申请
     * @param productVersion 产品版本
     * @param keyVersion     秘钥版本，照填即可
     * @param runMode        运行模式，0正常，1调试，开发阶段填1
     * @param sysPath        系统路径，SDK会在该目录下写入保证正常运行必需的配置信息
     * @param sysCapacity    系统路径下存储空间大小，SDK对该目录的存储空间要求小（最小大小：10K，建议大小：100K），SDK写入次数较少，读取次数较多
     * @param caPath         安卓需要提供CA证书，用于https请求
     */
    public native int initDevice(String Platform, byte[] license, String serialNumber, int productID, long productVersion, int keyVersion, int runMode,
                           String sysPath, long sysCapacity, String appid, String caPath);

    /**
     * 调用反初始化后不可再调用其他接口，除非再次init
     *
     * @return
     */
    public native int unInitDevice();

    /**
     * 获取SDK版本信息
     *
     * @return [0] main_verison [1] sub_version [2] fix_version [3] build_no
     */
    public static native int[] getDeviceSDKVersion();

    /**
     * 主动刷新动态二维码，当二维码失效时应该刷新一下
     * @return
     */
    static native int refreshQRCodeUrl();

    /**
     * 获取公共二维码
     * @return
     */
    static native int getBindTicket();

    /**
     * 获取设备UIN，可以显示一下
     * @return
     */
    static native long getUin();

    /**
     * 获取设备ID
     * @return
     */
    static native String getDeviceId();

    /**
     * 获取设备profile
     * @return
     */
    static native DeviceProfile getDeviceProfile();

    /**
     * 获取绑定列表
     *
     * @return 返回null时表示正在拉取，需要监听onBinderListChange以得到拉取结果
     */
    static native DeviceBinderInfo[] fetchBinderList();

    /**
     * 解绑所有绑定者（须在登录成功之后调用）
     *
     * @return
     */
    static native int eraseAllBinders();

    /**
     * 利用username解绑单个绑定者（须在登录成功之后调用）
     *
     * @return
     */
    static native int eraseBinder(String binderUserName);

    /**
     * 获取好友请求验证
     *
     * @return 未验证的所有好友请求
     */
    static native DeviceBinderInfo[] getUnverifyBinderList();

    /**
     * 处理好友请求验证
     *
     * @return 成功或者失败
     */
    static native int handleBinderVerify(String binderUserName, boolean isAccept);

    /**
     * 改好友别名
     *
     * @return 成功或者失败
     */
    static native int updateBinderAlias(String binderUserName, String alias);

    /**
     * 重置设备，设备所有信息将被清空，慎用
     * @return 0成功，其它失败
     */
    static native int factoryReset();

    /**
     * 改设备自己的昵称
     *
     * @return 成功或者失败
     */
    static native int updateDeviceNickname(String nickname);

    /**
     * 改设备自己的头像，请自己确保路径下的图片可用
     * @param jpgFilePath JPG格式的头像图片文件地址，注意是本文件，不是网络文件，512x512 或256x256
     *
     * @return 成功或者失败
     */
    static native int updateDeviceAvatar(String jpgFilePath);

    /**
     * 上传本地日志到服务器
     */
    static native void upLoadLog();

    /**
     * 设置设备场景
     * @param sceneType 场景类型，可以一次设置多个场景，可以为空数组
     * @param sceneInfo 场景类型对应的信息
     * @return
     */
    static native int setDeviceScene(int[] sceneType, String[] sceneInfo);

    /**
     * 设置声波配网的cacheKey
     * @return
     */
    static native int setCacheKey(String k, String v);  

    /**
     * 获取设备现在的登录状态
     *
     * @retval 0 logout
     * @retval 1 logining
     * @retval 2 login
     * @retval 3 unregister
     */
    static native int getAccountStatus();

    /**
     * 获取SDK现在的登录状态
     *
     * @retval 0 SDK is uninitialized
     * @retval 1 SDK initialization in progress
     * @retval 2 SDK initialize failed
     * @retval 3 SDK initialize success
     */
    static native int checkStatus();

    /**
     * 设置GPS回调
     */
    native static int setGpsCallback();

    /***   小微   ***/
    /**********语音识别核心请求 begin**********/

    static native int startXiaoweiService();

    static native int stopXiaoweiService();

//    static native void setXWAccountInfo(DeviceAccountInfo info);这个要放到device层

    static native String request(int type, byte[] requestData, XWRequestInfo param);

    static native int cancelRequest(String voiceID);

    static native void cancelTTS(String resId);

    /**********语音识别核心请求 end**********/

    static native void reportEvent(XWEventLogInfo log);

    static native int reportPlayState(XWPlayStateInfo stateInfo);

    static native int reportVolumeChange(int volume);

    static native int setUserState(XWPlayStateInfo stateInfo);

    static native int clearUserState();


    /**
     * 开启可见可答，实时取词
     *
     * @param enable true:开启 false:关闭
     * @return 0成功 非0失败
     */
    static native int enableV2A(boolean enable);

    /**
     * 设置词库列表
     *
     * @param type       词库类型
     * @param words_list 词库
     * @return 0:success else failed
     */
    static native int setWordslist(int type, String[] words_list);

    /**
     * 上传本地数据，一般用于本地技能
     *
     * @param type {XWCommonDef.DataListType}
     * @param words_list
     * @return 0:success else failed
     */
    static native String uploadDatalist(int type, String[] words_list);


    static native long reportUserLog(XWEventLogInfo log);

    /**
     * 拉取设备闹钟列表
     *
     * @return 请求sessionID
     */
    static native String getDeviceAlarmList();

    /**
     * 操作设备闹钟
     *
     * @param optType      操作类型（1：新增，2：更新，3：删除}
     * @param strAlarmJson 一个闹钟项的json串
     * @return 请求sessionID
     */
    static native String setDeviceAlarm(int optType, String strAlarmJson);

    /**
     * 拉取定时任务的播放资源
     *
     * @param strAlarmId 一个闹钟项的ID
     * @return 请求sessionID
     */
    static native String getTimingSkillResource(String strAlarmId);

    /**
     * 设置登录态，登录态是有屏设备用于获取音乐资源、语音点播歌曲、腾讯视频登录等使用的。QQ登录态90天会过期，微信30天过期，所以可能需要在设备端提示用户重新登录并设置。（过期是一直不使用才过期，每次使用自动续期）
     *
     * @param info
     * @return
     */
    static native String setXWLoginStatus(XWLoginStatusInfo info);

    /**
     * 查询登录态
     *
     * @param skillID
     * @return
     */
    static native XWLoginStatusInfo getXWLoginStatus(String skillID);


    /**
     * 请求指定格式的TTS，给视频通话、消息、导航等特殊场景使用
     *
     * @param tinyid    目标用户id，电话和消息需要填写
     * @param timestamp 时间 ,消息需要填，其余填0
     * @param type      类别
     * @return TTS的resId
     */
    static native String requestProtocolTTS(long tinyid, long timestamp, int type);

    /**
     * 发送音频消息，使用sendMessage代替
     *
     * @param msgId     动态消息ID，请到配置平台进行注册，没有注册过的ID的消息无法送到手机QQ
     * @param file_path 文件路径
     * @param duration  音频时长
     * @param targetIds 接收者的tinyid, 填空表示发送给所有绑定者
     * @return cookie，可以根据该cookie值在OnRichMsgSendProgress和OnRichMsgSendRet回调中查询进度信息和结果信息
     */
    static native long nativeSendAudioMsg(int msgId, String file_path, int duration, long[] targetIds);


    /**
     * 对上一次的结果不满意
     *
     * @param voiceID 不满意的voiceid
     */
    static native void errorFeedBack(String voiceID);

    /**
     * 发起命令类型的请求
     *
     * @param cmd cmdID，详见说明文档
     * @param subCmd subCMDID，详见说明文档
     * @param params 请求参数，详见说明文档
     */
    static native String requestCmd(String cmd, String subCmd, String params);

    /**
     * fetch iot list，会在通用resposne中返回响应
     */
    static native int getIotDeviceList();

    static native int setDeviceAvailability(int av);

     /**
     * 重置系统时钟
     * @return
     */
    static native int manulSetClock();

    /**
     * 查询设备睡眠模式
     * @return
     */
    static native int getSleepState();

    /**
     * 临时屏蔽5分钟睡眠模式
     * @return
     */
    static native int maskSleepModeTemporarily();

    /**
     * 获取本地闹钟列表
     * @return
     */
    static native String visitLocalClocks();

    /**
     * 获取云端ticket
     * @return
     */
    static native int getCloudTicket(DeviceSDK.CloudTicketListener listener);

    /**
     * 创建解码工具
     * @return
     */
    static native long decoderCreate(String key);

    /**
     * 开始解码
     * @return
     */
    static native void decoderDecode(long decoder , byte[] data,int size ,int offset);

    /**
     * 销毁解码工具
     * @return
     */
    static native void decoderDestroy(long decoder);
    
    /************************************ota begin************************************/

     static native int deviceConfigOta();

     static native int queryOtaUpdate(String businessID, int channel, int otaType, int debugFlag);

     static native int otaDwonload(String url, String downloadPath, int hashType, String hashValue);

     static native int stopOtaDownload(String url);


    /************************************ota end************************************/

    /************************************jni callback begin************************************/

    ///////////basic callback
    private void onLoginComplete(int error) {
        DeviceBaseManager.onLoginComplete(error);
    }

    private void onNetStatusChange(int newStatus) {
        DeviceBaseManager.onNetStatusChange(newStatus);
    }

    private void onNicNameChange(int error, String nickName) {
        DeviceBaseManager.onNicNameChange(error, nickName);
    }

    private void onAvatarChange(int error, String avatarUrl) {
        DeviceBaseManager.onAvatarChange(error, avatarUrl);
    }

    private void onLogoutComplete(int error) {
        DeviceBaseManager.onLogoutComplete(error);
    }

    private void onBinderListChange(int error, DeviceBinderInfo[] listBinder) {
        synchronized (mBinderList) {
            mBinderList.clear();
            for (int i = 0; listBinder != null && i < listBinder.length; ++i) {
                if (null == listBinder[i]) {
                    Log.d(TAG, "onBinderListChange: listBinder[" + i + "] is null ");
                    continue;
                }
                mBinderList.add(listBinder[i]);
                DeviceBaseManager.mAllFriendListCache.put(listBinder[i].userName, listBinder[i]);

                Log.d(TAG, "onBinderListChange: username = " + listBinder[i].userName + " , remark = "+ new String(listBinder[i].remark) + ",nickname = "+new String(listBinder[i].nickName) + " type = " + listBinder[i].type);
            }
        }
        DeviceBaseManager.onBinderListChange(error, getBinderList());
    }

    private void onBinderVerify(DeviceBinderInfo info) {
        DeviceBaseManager.onBinderVerify(info);
    }

    private void onQrCodeRefresh(String path, int expireTime, int expireIn) {
        DeviceBaseManager.onQrCodeRefresh(path, expireTime, expireIn);
    }

    private void onQrCodeScan(String userName, String avatar, int scanTime) {
        DeviceBaseManager.onQrCodeScan(userName, avatar, scanTime);
    }

    private void onTicketRefresh(String path, int expireTime, int expireIn) {
        DeviceBaseManager.onTicketRefresh(path, expireTime, expireIn);
    }

    private DeviceLocationInfo onGpsCallback() {
        return DeviceSDK.getInstance().onGpsJniCallback();
    }

    private void onGetSDKLog(int level, String tag, String filename, int line, String funcname, String message) {
         if (level == 0) {
             Log.v(tag, filename+" "+ line + " "+funcname + " "+message);
         } else if (level == 1) {
             Log.d(tag, filename+" "+ line + " "+funcname + " "+message);
         } else if (level == 2) {
             Log.i(tag, filename+" "+ line + " "+funcname + " "+message);
         } else if (level == 3) {
             Log.w(tag, filename+" "+ line + " "+funcname + " "+message);
         } else {
             Log.e(tag, filename+" "+ line + " "+funcname + " "+message);
         }

        DeviceBaseManager.onGetSDKLog(level, tag, filename, line, funcname, message);
    }


    private boolean onRequestCallback(String voiceID, int event, XWResponseInfo rspData, byte[] extendData) {
        return DeviceSDK.getInstance().onRequest(voiceID, event, rspData, extendData);
    }

    //通用请求回调
    private void OnCommonRequest(String voiceId, int error, String json) {
        DeviceSDK.getInstance().onCommonRequest(voiceId, error, json);
    }

    private void onUploadData(String voiceId, int errCode) {
        DeviceSDK.getInstance().onUploadData(voiceId, errCode);
    }

    private boolean onNetworkDelayCallback(String voiceID, long time) {
        return DeviceSDK.getInstance().onNetWorkDelay(voiceID, time);
    }

    @Deprecated
    private boolean onTTSPushCallback(String voiceID, XWTTSDataInfo tts) {
        return DeviceSDK.getInstance().onTTSPushData(voiceID, tts);
    }

    //ota
    private void onOtaCheckResult(int errCodde, String businessID, boolean newVersion, DeviceOTAInfo otaInfo) {
        DeviceSDK.getInstance().onOtaCheckResult(errCodde,businessID,newVersion,otaInfo);
    }


    private void onOtaDownLoadProgress(String url, double current, double total) {
        DeviceSDK.getInstance().onOtaDownLoadProgress(url,current,total);
    }

    private void onOtaDownLoadError(String url, int errCode) {
        DeviceSDK.getInstance().onOtaDownLoadError(url,errCode);
    }

    private void onOtaDownLoadComplete(String url, String filePath) {
        DeviceSDK.getInstance().onOtaDownLoadComplete(url,filePath);
    }

    private void onReceiveWifiInfo(String ssid, String pwd, String token) {
        XWVoiceLinkManager.onReceiveWifiInfo(ssid, pwd, token);
    }

    /************************************jni callback end************************************/


    public static ArrayList<DeviceBinderInfo> getBinderList() {
        synchronized (mBinderList) {
            ArrayList<DeviceBinderInfo> list = new ArrayList<>(mBinderList.size());
            for (DeviceBinderInfo info : mBinderList) {
                try {
                    list.add((DeviceBinderInfo) info.clone());
                } catch (CloneNotSupportedException e) {
                    e.printStackTrace();
                }
            }
            return list;
        }
    }


    public int destroy() {
        mBinderList.clear();
        mLogMap.clear();
        DeviceBaseManager.mAllFriendListCache.clear();
        return unInitDevice();
    }


}
